@@ -573,12 +573,12 @@ repGuards other
            = do { x <- repE e1; y <- repE e2; return (x, y) }
     process other = panic "Non Haskell 98 guarded body"
 
-repFields :: [(Name,HsExpr Name)] -> DsM (Core [M.FieldExp])
+repFields :: [(Name, HsExpr Name)] -> DsM (Core [M.Q M.FieldExp])
 repFields flds = do
         fnames <- mapM lookupOcc (map fst flds)
         es <- mapM repE (map snd flds)
-        fs <- zipWithM (\n x -> rep2 fieldExpName [unC n, unC x]) fnames es
-        coreList fieldExpTyConName fs
+        fs <- zipWithM repFieldExp fnames es
+        coreList fieldExpQTyConName fs
 
 
 -----------------------------------------------------------------------------
@@ -677,10 +677,11 @@ rep_monobind' (AndMonoBinds x y) = do { x1 <- rep_monobind' x;
 rep_monobind' (FunMonoBind fn infx [Match [] ty (GRHSs guards wheres ty2)] loc) 
  = do { (ss,wherecore) <- repBinds wheres
 	; guardcore <- addBinds ss (repGuards guards)
-	; fn' <- lookupBinder fn
-	; p   <- repPvar fn'
-	; ans <- repVal p guardcore wherecore
-	; return [(loc, ans)] }
+	; fn'  <- lookupBinder fn
+	; p    <- repPvar fn'
+	; ans  <- repVal p guardcore wherecore
+	; ans' <- wrapGenSyns ss ans
+	; return [(loc, ans')] }
 
 rep_monobind' (FunMonoBind fn infx ms loc)
  =   do { ms1 <- mapM repClauseTup ms
@@ -692,8 +693,9 @@ rep_monobind' (PatMonoBind pat (GRHSs guards wheres ty2) loc)
  =   do { patcore <- repP pat 
         ; (ss,wherecore) <- repBinds wheres
 	; guardcore <- addBinds ss (repGuards guards)
-        ; ans <- repVal patcore guardcore wherecore
-        ; return [(loc, ans)] }
+        ; ans  <- repVal patcore guardcore wherecore
+	; ans' <- wrapGenSyns ss ans
+        ; return [(loc, ans')] }
 
 rep_monobind' (VarMonoBind v e)  
  =   do { v' <- lookupBinder v 
@@ -991,12 +993,15 @@ repListExp (MkC es) = rep2 listEName [es]
 repSigExp :: Core M.ExpQ -> Core M.TypeQ -> DsM (Core M.ExpQ)
 repSigExp (MkC e) (MkC t) = rep2 sigEName [e,t]
 
-repRecCon :: Core String -> Core [M.FieldExp]-> DsM (Core M.ExpQ)
-repRecCon (MkC c) (MkC fs) = rep2 recCName [c,fs]
+repRecCon :: Core String -> Core [M.Q M.FieldExp]-> DsM (Core M.ExpQ)
+repRecCon (MkC c) (MkC fs) = rep2 recConEName [c,fs]
 
-repRecUpd :: Core M.ExpQ -> Core [M.FieldExp] -> DsM (Core M.ExpQ)
+repRecUpd :: Core M.ExpQ -> Core [M.Q M.FieldExp] -> DsM (Core M.ExpQ)
 repRecUpd (MkC e) (MkC fs) = rep2 recUpdEName [e,fs]
 
+repFieldExp :: Core String -> Core M.ExpQ -> DsM (Core (M.Q M.FieldExp))
+repFieldExp (MkC n) (MkC x) = rep2 fieldExpName [n,x]
+
 repInfixApp :: Core M.ExpQ -> Core M.ExpQ -> Core M.ExpQ -> DsM (Core M.ExpQ)
 repInfixApp (MkC x) (MkC y) (MkC z) = rep2 infixAppName [x,y,z]
 
@@ -1270,7 +1275,7 @@ templateHaskellNames = mkNameSet [
     clauseQTyConName, expQTyConName, fieldExpTyConName, stmtQTyConName,
     decQTyConName, conQTyConName, strictTypeQTyConName,
     varStrictTypeQTyConName, typeQTyConName, expTyConName, decTyConName,
-    typeTyConName, matchTyConName, clauseTyConName]
+    typeTyConName, matchTyConName, clauseTyConName, fieldExpQTyConName]
 
 varQual  = mk_known_key_name OccName.varName
 tcQual   = mk_known_key_name OccName.tcName
@@ -1413,8 +1418,9 @@ decTyConName      = tcQual  FSLIT("Dec")          decTyConKey
 typeTyConName     = tcQual  FSLIT("Type")         typeTyConKey
 matchTyConName    = tcQual  FSLIT("Match")        matchTyConKey
 clauseTyConName   = tcQual  FSLIT("Clause")       clauseTyConKey
+fieldExpQTyConName = tcQual FSLIT("FieldExpQ")      fieldExpQTyConKey
 
---	TyConUniques available: 100-119
+--	TyConUniques available: 100-129
 -- 	Check in PrelNames if you want to change this
 
 expTyConKey             = mkPreludeTyConUnique 100
@@ -1435,6 +1441,7 @@ varStrictTypeQTyConKey  = mkPreludeTyConUnique 114
 strictTypeQTyConKey     = mkPreludeTyConUnique 115
 fieldExpTyConKey        = mkPreludeTyConUnique 116
 fieldPatTyConKey        = mkPreludeTyConUnique 117
+fieldExpQTyConKey       = mkPreludeTyConUnique 118
 
 -- 	IdUniques available: 200-299
 -- 	If you want to change this, make sure you check in PrelNames
