@@ -1,10 +1,10 @@
 {-# OPTIONS -#include "Linker.h" #-}
 -----------------------------------------------------------------------------
--- $Id: InteractiveUI.hs,v 1.159.2.1 2003/11/19 10:01:20 simonmar Exp $
+-- $Id: InteractiveUI.hs,v 1.159.2.7 2004/10/13 14:01:37 simonmar Exp $
 --
 -- GHC Interactive User Interface
 --
--- (c) The GHC Team 2000
+-- (c) The GHC Team 2004
 --
 -----------------------------------------------------------------------------
 module InteractiveUI ( 
@@ -52,7 +52,7 @@ import System.Posix
 #endif
 #endif
 
-#if HAVE_READLINE_HEADERS && HAVE_READLINE_LIBS
+#ifdef USE_READLINE
 import Control.Concurrent	( yield )	-- Used in readline loop
 import System.Console.Readline as Readline
 #endif
@@ -65,6 +65,7 @@ import Control.Concurrent
 
 import Numeric
 import Data.List
+import Data.Int		( Int64 )
 import System.Cmd
 import System.CPUTime
 import System.Environment
@@ -72,6 +73,7 @@ import System.Directory
 import System.IO as IO
 import Data.Char
 import Control.Monad as Monad
+import Foreign.StablePtr	( newStablePtr )
 
 import GHC.Exts		( unsafeCoerce# )
 
@@ -81,12 +83,12 @@ import System.Posix.Internals ( setNonBlockingFD )
 
 -----------------------------------------------------------------------------
 
-ghciWelcomeMsg = "\ 
-\   ___         ___ _\n\ 
-\  / _ \\ /\\  /\\/ __(_)\n\ 
-\ / /_\\// /_/ / /  | |      GHC Interactive, version " ++ cProjectVersion ++ ", for Haskell 98.\n\ 
-\/ /_\\\\/ __  / /___| |      http://www.haskell.org/ghc/\n\ 
-\\\____/\\/ /_/\\____/|_|      Type :? for help.\n"
+ghciWelcomeMsg =
+ "   ___         ___ _\n" ++
+ "  / _ \\ /\\  /\\/ __(_)\n" ++
+ " / /_\\// /_/ / /  | |      GHC Interactive, version " ++ cProjectVersion ++ ", for Haskell 98.\n" ++
+ "/ /_\\\\/ __  / /___| |      http://www.haskell.org/ghc/\n" ++
+ "\\____/\\/ /_/\\____/|_|      Type :? for help.\n"
 
 GLOBAL_VAR(commands, builtin_commands, [(String, String -> GHCi Bool)])
 
@@ -119,41 +121,40 @@ keepGoingPaths a str = a (toArgs str) >> return False
 shortHelpText = "use :? for help.\n"
 
 -- NOTE: spaces at the end of each line to workaround CPP/string gap bug.
-helpText = "\ 
-\ Commands available from the prompt:\n\ 
-\\n\ 
-\   <stmt>		       evaluate/run <stmt>\n\ 
-\   :add <filename> ...        add module(s) to the current target set\n\ 
-\   :browse [*]<module>	       display the names defined by <module>\n\ 
-\   :cd <dir>		       change directory to <dir>\n\ 
-\   :def <cmd> <expr>          define a command :<cmd>\n\ 
-\   :help, :?		       display this list of commands\n\ 
-\   :info [<name> ...]         display information about the given names\n\ 
-\   :load <filename> ...       load module(s) and their dependents\n\ 
-\   :module [+/-] [*]<mod> ... set the context for expression evaluation\n\ 
-\   :reload		       reload the current module set\n\ 
-\\n\ 
-\   :set <option> ...	       set options\n\ 
-\   :set args <arg> ...	       set the arguments returned by System.getArgs\n\ 
-\   :set prog <progname>       set the value returned by System.getProgName\n\ 
-\\n\ 
-\   :show modules	       show the currently loaded modules\n\ 
-\   :show bindings	       show the current bindings made at the prompt\n\ 
-\\n\ 
-\   :type <expr>	       show the type of <expr>\n\ 
-\   :undef <cmd> 	       undefine user-defined command :<cmd>\n\ 
-\   :unset <option> ...	       unset options\n\ 
-\   :quit		       exit GHCi\n\ 
-\   :!<command>		       run the shell command <command>\n\ 
-\\n\ 
-\ Options for `:set' and `:unset':\n\ 
-\\n\ 
-\    +r			revert top-level expressions after each evaluation\n\ 
-\    +s                 print timing/memory stats after each evaluation\n\ 
-\    +t			print type after evaluation\n\ 
-\    -<flags>		most GHC command line flags can also be set here\n\ 
-\                         (eg. -v2, -fglasgow-exts, etc.)\n\ 
-\"
+helpText =
+ " Commands available from the prompt:\n" ++
+ "\n" ++
+ "   <stmt>		       evaluate/run <stmt>\n" ++
+ "   :add <filename> ...        add module(s) to the current target set\n" ++
+ "   :browse [*]<module>	       display the names defined by <module>\n" ++
+ "   :cd <dir>		       change directory to <dir>\n" ++
+ "   :def <cmd> <expr>          define a command :<cmd>\n" ++
+ "   :help, :?		       display this list of commands\n" ++
+ "   :info [<name> ...]         display information about the given names\n" ++
+ "   :load <filename> ...       load module(s) and their dependents\n" ++
+ "   :module [+/-] [*]<mod> ... set the context for expression evaluation\n" ++
+ "   :reload		       reload the current module set\n" ++
+ "\n" ++
+ "   :set <option> ...	       set options\n" ++
+ "   :set args <arg> ...	       set the arguments returned by System.getArgs\n" ++
+ "   :set prog <progname>       set the value returned by System.getProgName\n" ++
+ "\n" ++
+ "   :show modules	       show the currently loaded modules\n" ++
+ "   :show bindings	       show the current bindings made at the prompt\n" ++
+ "\n" ++
+ "   :type <expr>	       show the type of <expr>\n" ++
+ "   :undef <cmd> 	       undefine user-defined command :<cmd>\n" ++
+ "   :unset <option> ...	       unset options\n" ++
+ "   :quit		       exit GHCi\n" ++
+ "   :!<command>		       run the shell command <command>\n" ++
+ "\n" ++
+ " Options for `:set' and `:unset':\n" ++
+ "\n" ++
+ "    +r			revert top-level expressions after each evaluation\n" ++
+ "    +s                 print timing/memory stats after each evaluation\n" ++
+ "    +t			print type after evaluation\n" ++
+ "    -<flags>		most GHC command line flags can also be set here\n" ++
+ "                         (eg. -v2, -fglasgow-exts, etc.)\n"
 
 interactiveUI :: [FilePath] -> Maybe String -> IO ()
 interactiveUI srcs maybe_expr = do
@@ -161,6 +162,18 @@ interactiveUI srcs maybe_expr = do
 
    cmstate <- cmInit Interactive;
 
+   -- HACK! If we happen to get into an infinite loop (eg the user
+   -- types 'let x=x in x' at the prompt), then the thread will block
+   -- on a blackhole, and become unreachable during GC.  The GC will
+   -- detect that it is unreachable and send it the NonTermination
+   -- exception.  However, since the thread is unreachable, everything
+   -- it refers to might be finalized, including the standard Handles.
+   -- This sounds like a bug, but we don't have a good solution right
+   -- now.
+   newStablePtr stdin
+   newStablePtr stdout
+   newStablePtr stderr
+
    hFlush stdout
    hSetBuffering stdout NoBuffering
 
@@ -174,7 +187,7 @@ interactiveUI srcs maybe_expr = do
 	-- initial context is just the Prelude
    cmstate <- cmSetContext cmstate dflags [] ["Prelude"]
 
-#if HAVE_READLINE_HEADERS && HAVE_READLINE_LIBS
+#ifdef USE_READLINE
    Readline.initialize
 #endif
 
@@ -185,7 +198,7 @@ interactiveUI srcs maybe_expr = do
 		   cmstate = cmstate,
 		   options = [] }
 
-#if HAVE_READLINE_HEADERS && HAVE_READLINE_LIBS
+#ifdef USE_READLINE
    Readline.resetTerminal Nothing
 #endif
 
@@ -254,7 +267,7 @@ interactiveLoop is_tty show_prompt = do
 			_other      -> return ()) $ do
 
   -- read commands from stdin
-#if HAVE_READLINE_HEADERS && HAVE_READLINE_LIBS
+#ifdef USE_READLINE
   if (is_tty) 
 	then readlineLoop
 	else fileLoop stdin show_prompt
@@ -320,7 +333,7 @@ stringLoop (s:ss) = do
 mkPrompt toplevs exports
    = concat (intersperse " " (map ('*':) toplevs ++ exports)) ++ "> "
 
-#if HAVE_READLINE_HEADERS && HAVE_READLINE_LIBS
+#ifdef USE_READLINE
 readlineLoop :: GHCi ()
 readlineLoop = do
    cmstate <- getCmState
@@ -416,7 +429,7 @@ specialCommand str = do
   let (cmd,rest) = break isSpace str
   cmds <- io (readIORef commands)
   case [ (s,f) | (s,f) <- cmds, prefixMatch cmd s ] of
-     []      -> io (hPutStr stdout ("unknown command `:" ++ cmd ++ "'\n" 
+     []      -> io (hPutStr stdout ("unknown command ':" ++ cmd ++ "'\n" 
 		                    ++ shortHelpText) >> return False)
      [(_,f)] -> f (dropWhile isSpace rest)
      cs      -> io (hPutStrLn stdout ("prefix " ++ cmd ++ 
@@ -424,7 +437,7 @@ specialCommand str = do
 	         	     	       foldr1 (\a b -> a ++ ',':b) (map fst cs)
 					 ++ ")") >> return False)
 
-noArgs c = throwDyn (CmdLineError ("command `" ++ c ++ "' takes no arguments"))
+noArgs c = throwDyn (CmdLineError ("command '" ++ c ++ "' takes no arguments"))
 
 
 -----------------------------------------------------------------------------
@@ -548,8 +561,8 @@ changeDirectory :: String -> GHCi ()
 changeDirectory dir = do
   state    <- getGHCiState
   when (targets state /= []) $
-	io $ putStr "Warning: changing directory causes all loaded modules to be unloaded, \n\ 
-	\because the search path has changed.\n"
+	io $ putStr ("Warning: changing directory causes all loaded modules to be unloaded, \n" ++
+	"because the search path has changed.\n")
   dflags   <- io getDynFlags
   cmstate1 <- io (cmUnload (cmstate state) dflags)
   setGHCiState state{ cmstate = cmstate1, targets = [] }
@@ -566,7 +579,7 @@ defineMacro s = do
 	else do
   if (macro_name `elem` map fst cmds) 
 	then throwDyn (CmdLineError 
-		("command `" ++ macro_name ++ "' is already defined"))
+		("command '" ++ macro_name ++ "' is already defined"))
 	else do
 
   -- give the expression a type signature, so we can be sure we're getting
@@ -593,11 +606,11 @@ undefineMacro macro_name = do
   cmds <- io (readIORef commands)
   if (macro_name `elem` map fst builtin_commands) 
 	then throwDyn (CmdLineError
-		("command `" ++ macro_name ++ "' cannot be undefined"))
+		("command '" ++ macro_name ++ "' cannot be undefined"))
 	else do
   if (macro_name `notElem` map fst cmds) 
 	then throwDyn (CmdLineError 
-		("command `" ++ macro_name ++ "' not defined"))
+		("command '" ++ macro_name ++ "' not defined"))
 	else do
   io (writeIORef commands (filter ((/= macro_name) . fst) cmds))
 
@@ -700,7 +713,7 @@ browseModule m exports_only = do
 
   is_interpreted <- io (cmModuleIsInterpreted cms m)
   when (not is_interpreted && not exports_only) $
-	throwDyn (CmdLineError ("module `" ++ m ++ "' is not interpreted"))
+	throwDyn (CmdLineError ("module '" ++ m ++ "' is not interpreted"))
 
   -- temporarily set the context to the module we're interested in,
   -- just so we can get an appropriate PrintUnqualified
@@ -774,7 +787,7 @@ separate cmstate []           as bs = return (as,bs)
 separate cmstate (('*':m):ms) as bs = do
    b <- io (cmModuleIsInterpreted cmstate m)
    if b then separate cmstate ms (m:as) bs
-   	else throwDyn (CmdLineError ("module `" ++ m ++ "' is not interpreted"))
+   	else throwDyn (CmdLineError ("module '" ++ m ++ "' is not interpreted"))
 separate cmstate (m:ms)       as bs = separate cmstate ms as (m:bs)
 
 prel = "Prelude"
@@ -885,7 +898,7 @@ unsetOptions str
 	   (plus_opts, rest2)  = partition isPlus rest1
 
        if (not (null rest2)) 
-	  then io (putStrLn ("unknown option: `" ++ head rest2 ++ "'"))
+	  then io (putStrLn ("unknown option: '" ++ head rest2 ++ "'"))
 	  else do
 
        mapM_ unsetOpt plus_opts
@@ -903,12 +916,12 @@ isPlus _ = False
 
 setOpt ('+':str)
   = case strToGHCiOpt str of
-	Nothing -> io (putStrLn ("unknown option: `" ++ str ++ "'"))
+	Nothing -> io (putStrLn ("unknown option: '" ++ str ++ "'"))
 	Just o  -> setOption o
 
 unsetOpt ('+':str)
   = case strToGHCiOpt str of
-	Nothing -> io (putStrLn ("unknown option: `" ++ str ++ "'"))
+	Nothing -> io (putStrLn ("unknown option: '" ++ str ++ "'"))
 	Just o  -> unsetOption o
 
 strToGHCiOpt :: String -> (Maybe GHCiOption)
@@ -1051,18 +1064,20 @@ timeIt action
 		  a <- action
 		  allocs2 <- io $ getAllocations
 		  time2   <- io $ getCPUTime
-		  io $ printTimes (allocs2 - allocs1) (time2 - time1)
+		  io $ printTimes (fromIntegral (allocs2 - allocs1)) 
+				  (time2 - time1)
 		  return a
 
-foreign import ccall "getAllocations" getAllocations :: IO Int
+foreign import ccall unsafe "getAllocations" getAllocations :: IO Int64
+	-- defined in ghc/rts/Stats.c
 
-printTimes :: Int -> Integer -> IO ()
+printTimes :: Integer -> Integer -> IO ()
 printTimes allocs psecs
    = do let secs = (fromIntegral psecs / (10^12)) :: Float
 	    secs_str = showFFloat (Just 2) secs
 	putStrLn (showSDoc (
 		 parens (text (secs_str "") <+> text "secs" <> comma <+> 
-			 int allocs <+> text "bytes")))
+			 text (show allocs) <+> text "bytes")))
 
 -----------------------------------------------------------------------------
 -- reverting CAFs
