@@ -254,12 +254,11 @@ static_flags =
 				    then do writeIORef v_Split_object_files True
 					    add v_Opt_C "-fglobalise-toplev-names"
 				    else hPutStrLn stderr
-					    "warning: don't know how to  split \ 
-					    \object files on this architecture"
+					    "warning: don't know how to split object files on this architecture"
 				) )
 
 	------- Include/Import Paths ----------------------------------------
-  ,  ( "i"		, OptPrefix (addToDirList v_Import_paths) )
+  ,  ( "i"		, OptPrefix (addToOrDeleteDirList v_Import_paths) )
   ,  ( "I" 		, Prefix    (addToDirList v_Include_paths) )
 
 	------- Libraries ---------------------------------------------------
@@ -452,6 +451,7 @@ fFlags = [
   ( "warn-unused-imports",       	Opt_WarnUnusedImports ),
   ( "warn-unused-matches",       	Opt_WarnUnusedMatches ),
   ( "warn-deprecations",         	Opt_WarnDeprecations ),
+  ( "warn-misc",	         	Opt_WarnMisc ),
   ( "fi",				Opt_FFI ),  -- support `-ffi'...
   ( "ffi",				Opt_FFI ),  -- ...and also `-fffi'
   ( "arrows",				Opt_Arrows ), -- arrow syntax
