@@ -1,7 +1,7 @@
 {-# OPTIONS -fno-warn-incomplete-patterns -optc-DNON_POSIX_SOURCE #-}
 
 -----------------------------------------------------------------------------
--- $Id: Main.hs,v 1.132.2.3 2004/02/02 10:19:04 simonmar Exp $
+-- $Id: Main.hs,v 1.132.2.4 2004/08/31 13:53:23 simonmar Exp $
 --
 -- GHC Driver program
 --
@@ -99,8 +99,7 @@ main =
 		-- an IO exception probably isn't our fault, so don't panic
 		IOException _ ->  hPutStrLn stderr (show exception)
 		AsyncException StackOverflow ->
-			hPutStrLn stderr "stack overflow: use +RTS -K<size> \ 
-					 \to increase it"
+			hPutStrLn stderr "stack overflow: use +RTS -K<size> to increase it"
 		_other ->  hPutStr stderr (show (Panic (show exception)))
 	   exitWith (ExitFailure 1)
          ) $ do
