@@ -1,5 +1,5 @@
 /* ----------------------------------------------------------------------------
- * $Id: ClosureTypes.h,v 1.16 2001/07/23 17:23:19 simonmar Exp $
+ * $Id: ClosureTypes.h,v 1.18 2002/12/11 15:36:37 simonmar Exp $
  * 
  * (c) The GHC Team, 1998-1999
  *
@@ -40,48 +40,45 @@
 #define THUNK_STATIC	        24
 #define THUNK_SELECTOR	        25
 #define BCO		        26
-#define AP_UPD		        27
+#define AP		        27
 #define PAP			28
-#define IND		        29
-#define IND_OLDGEN	        30
-#define IND_PERM	        31
-#define IND_OLDGEN_PERM	        32
-#define IND_STATIC	        33
-#define RET_BCO                 36
-#define RET_SMALL	        37
-#define RET_VEC_SMALL	        38
-#define RET_BIG		        39
-#define RET_VEC_BIG	        40
-#define RET_DYN		        41
+#define AP_STACK                29
+#define IND		        30
+#define IND_OLDGEN	        31
+#define IND_PERM	        32
+#define IND_OLDGEN_PERM	        33
+#define IND_STATIC	        34
+#define RET_BCO                 35
+#define RET_SMALL	        36
+#define RET_VEC_SMALL	        37
+#define RET_BIG		        38
+#define RET_VEC_BIG	        39
+#define RET_DYN		        40
+#define RET_FUN                 41
 #define UPDATE_FRAME	        42
 #define CATCH_FRAME	        43
 #define STOP_FRAME	        44
-#define SEQ_FRAME	        45
-#define CAF_BLACKHOLE		46
-#define BLACKHOLE	        47
-#define BLACKHOLE_BQ	        48
-#define SE_BLACKHOLE		49
-#define SE_CAF_BLACKHOLE	50
-#define MVAR		        51
-#define ARR_WORDS	        52
-#define MUT_ARR_PTRS	        53
-#define MUT_ARR_PTRS_FROZEN     54
-#define MUT_VAR		        55
-#define MUT_CONS                56
-#define WEAK		        57
-#define FOREIGN		        58
-#define STABLE_NAME	        59
-
-#define TSO		        60
-#define BLOCKED_FETCH	        61
-#define FETCH_ME                62
-#define FETCH_ME_BQ             63
-#define RBH                     64
-
-#define EVACUATED               65
-
-#define REMOTE_REF              66
-
-#define N_CLOSURE_TYPES         67
-
+#define CAF_BLACKHOLE		45
+#define BLACKHOLE	        46
+#define BLACKHOLE_BQ	        47
+#define SE_BLACKHOLE		48
+#define SE_CAF_BLACKHOLE	49
+#define MVAR		        50
+#define ARR_WORDS	        51
+#define MUT_ARR_PTRS	        52
+#define MUT_ARR_PTRS_FROZEN     53
+#define MUT_VAR		        54
+#define MUT_CONS                55
+#define WEAK		        56
+#define FOREIGN		        57
+#define STABLE_NAME	        58
+#define TSO		        59
+#define BLOCKED_FETCH	        60
+#define FETCH_ME                61
+#define FETCH_ME_BQ             62
+#define RBH                     63
+#define EVACUATED               64
+#define REMOTE_REF              65
+#define N_CLOSURE_TYPES         66
+				
 #endif /* CLOSURETYPES_H */
