@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: FrontPanel.c,v 1.6.4.1 2001/12/13 13:39:46 simonmar Exp $
+ * $Id: FrontPanel.c,v 1.8.2.2 2004/09/07 08:16:57 simonmar Exp $
  *
  * (c) The GHC Team 2000
  *
@@ -397,6 +397,7 @@ updateFrontPanel( void )
 			    block_width, block_height, blocks_per_line );
 	    }
 	    
+#if 0 /* Segfaults because bd appears to be bogus but != NULL. stolz, 2003-06-24 */
 	    /* color each block */
 	    for (; a <= LAST_BLOCK(m); (char *)a += BLOCK_SIZE) {
 		bd = Bdescr((P_)a);
@@ -409,6 +410,7 @@ updateFrontPanel( void )
 				block_width, block_height, blocks_per_line );
 		}
 	    }
+#endif
 	}
 
 	
@@ -673,7 +675,7 @@ residencyCensus( void )
 			type = BlackHole;
 			break;
 
-		    case AP_UPD:
+		    case AP:
 			size = pap_sizeW((StgPAP *)p);
 			type = Thunk;
 			break;
