@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: Main.c,v 1.30.4.1 2001/09/05 16:32:24 sewardj Exp $
+ * $Id: Main.c,v 1.39.2.1 2003/11/10 12:02:49 simonmar Exp $
  *
  * (c) The GHC Team 1998-2000
  *
@@ -13,9 +13,11 @@
 #include "Rts.h"
 #include "RtsAPI.h"
 #include "SchedAPI.h"
+#include "Schedule.h"
 #include "RtsFlags.h"
 #include "RtsUtils.h"
 #include "Prelude.h"
+#include <stdlib.h>
 
 #ifdef DEBUG
 # include "Printer.h"   /* for printing        */
@@ -39,7 +41,7 @@
 # include <windows.h>
 #endif
 
-extern void __stginit_PrelMain(void);
+extern void __stginit_ZCMain(void);
 
 /* Hack: we assume that we're building a batch-mode system unless 
  * INTERPRETER is set
@@ -51,7 +53,7 @@ int main(int argc, char *argv[])
     SchedulerStatus status;
     /* all GranSim/GUM init is done in startupHaskell; sets IAmMainThread! */
 
-    startupHaskell(argc,argv,__stginit_PrelMain);
+    startupHaskell(argc,argv,__stginit_ZCMain);
 
     /* kick off the computation by creating the main thread with a pointer
        to mainIO_closure representing the computation of the overall program;
@@ -83,7 +85,7 @@ int main(int argc, char *argv[])
 		   fprintf(stderr, "==== [%x] Main Thread Started ...\n", mytid));
 
       /* ToDo: Dump event for the main thread */
-      status = rts_evalIO((HaskellObj)mainIO_closure, NULL);
+      status = rts_mainLazyIO((HaskellObj)mainIO_closure, NULL);
     } else {
       /* Just to show we're alive */
       IF_PAR_DEBUG(verbose,
@@ -98,21 +100,19 @@ int main(int argc, char *argv[])
 #  elif defined(GRAN)
 
     /* ToDo: Dump event for the main thread */
-    status = rts_evalIO(mainIO_closure, NULL);
+    status = rts_mainLazyIO(mainIO_closure, NULL);
 
 #  else /* !PAR && !GRAN */
 
     /* ToDo: want to start with a larger stack size */
-    status = rts_evalIO((HaskellObj)mainIO_closure, NULL);
+    rts_lock();
+    status = rts_evalLazyIO((HaskellObj)mainIO_closure, NULL);
+    rts_unlock();
 
 #  endif /* !PAR && !GRAN */
 
     /* check the status of the entire Haskell computation */
     switch (status) {
-    case Deadlock:
-      prog_belch("no threads to run:  infinite loop or deadlock?");
-      exit_status = EXIT_DEADLOCK;
-      break;
     case Killed:
       prog_belch("main thread exited (uncaught exception)");
       exit_status = EXIT_KILLED;
