@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: Printer.c,v 1.60.6.1 2003/10/20 11:24:00 simonmar Exp $
+ * $Id: Printer.c,v 1.60.6.2 2004/09/13 17:21:17 sof Exp $
  *
  * (c) The GHC Team, 1994-2000.
  *
@@ -354,6 +354,13 @@ printClosure( StgClosure *obj )
           }
 #endif  
 
+    case MVAR:
+        {
+	  StgMVar* mv = (StgMVar*)obj;
+	  fprintf(stderr,"MVAR(head=%p, link=%p, tail=%p, value=%p)\n", mv->head, mv->mut_link, mv->tail, mv->value);
+          break;
+        }
+
     case MUT_VAR:
         {
 	  StgMutVar* mv = (StgMutVar*)obj;
