@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: Printer.h,v 1.5 2000/01/14 14:56:40 simonmar Exp $
+ * $Id: Printer.h,v 1.7 2002/12/11 15:36:45 simonmar Exp $
  *
  * (c) The GHC Team, 1998-2000
  *
@@ -14,8 +14,6 @@ extern void   	   printObj        ( StgClosure *obj );
 extern void   	   printClosure    ( StgClosure *obj );
 extern StgStackPtr printStackObj   ( StgStackPtr sp );
 extern void        printStackChunk ( StgStackPtr sp, StgStackPtr spLim );
-extern void        printStack      ( StgStackPtr sp, StgStackPtr spLim, 
-				     StgUpdateFrame* su );
 extern void        printTSO        ( StgTSO *tso );
 
 void   	           info_hdr_type   ( StgClosure *closure, char *res );
@@ -24,5 +22,5 @@ char  *	           info_type_by_ip ( StgInfoTable *ip );
 
 extern void DEBUG_LoadSymbols( char *name );
 
-extern rtsBool lookupGHCName( StgPtr addr, const char **result );
+extern const char *lookupGHCName( void *addr );
 #endif
