@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: ProfHeap.c,v 1.47.2.2 2003/11/03 09:54:35 simonmar Exp $
+ * $Id: ProfHeap.c,v 1.47.2.5 2004/07/21 10:37:42 simonmar Exp $
  *
  * (c) The GHC Team, 1998-2003
  *
@@ -32,6 +32,7 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <math.h>
 
 /* -----------------------------------------------------------------------------
  * era stores the current time period.  It is the same as the
@@ -382,6 +383,16 @@ void endProfiling( void )
 }
 #endif /* DEBUG_HEAP_PROF */
 
+static void
+printSample(rtsBool beginSample, StgDouble sampleValue)
+{
+    StgDouble fractionalPart, integralPart;
+    fractionalPart = modf(sampleValue, &integralPart);
+    fprintf(hp_file, "%s %d.%02d\n",
+            (beginSample ? "BEGIN_SAMPLE" : "END_SAMPLE"),
+            (int)integralPart, (int)(fractionalPart * 100 + 0.5));
+}
+
 /* --------------------------------------------------------------------------
  * Initialize the heap profilier
  * ----------------------------------------------------------------------- */
@@ -430,10 +441,9 @@ initHeapProfiling(void)
 	int count;
 	for(count = 1; count < prog_argc; count++)
 	    fprintf(hp_file, " %s", prog_argv[count]);
-	fprintf(hp_file, " +RTS ");
+	fprintf(hp_file, " +RTS");
 	for(count = 0; count < rts_argc; count++)
-	    fprintf(hp_file, "%s ", rts_argv[count]);
-	fprintf(hp_file, "\n");
+	    fprintf(hp_file, " %s", rts_argv[count]);
     }
 #endif /* PROFILING */
 
@@ -444,8 +454,8 @@ initHeapProfiling(void)
     fprintf(hp_file, "SAMPLE_UNIT \"seconds\"\n");
     fprintf(hp_file, "VALUE_UNIT \"bytes\"\n");
 
-    fprintf(hp_file, "BEGIN_SAMPLE 0.00\n");
-    fprintf(hp_file, "END_SAMPLE 0.00\n");
+    printSample(rtsTrue, 0);
+    printSample(rtsFalse, 0);
 
 #ifdef DEBUG_HEAP_PROF
     DEBUG_LoadSymbols(prog_name);
@@ -487,8 +497,8 @@ endHeapProfiling(void)
 #endif
 
     seconds = mut_user_time();
-    fprintf(hp_file, "BEGIN_SAMPLE %0.2f\n", seconds);
-    fprintf(hp_file, "END_SAMPLE %0.2f\n", seconds);
+    printSample(rtsTrue, seconds);
+    printSample(rtsFalse, seconds);
     fclose(hp_file);
 }
 
@@ -584,6 +594,7 @@ rtsBool
 closureSatisfiesConstraints( StgClosure* p )
 {
 #ifdef DEBUG_HEAP_PROF
+    (void)p;   /* keep gcc -Wall happy */
     return rtsTrue;
 #else
    rtsBool b;
@@ -730,7 +741,7 @@ dumpCensus( Census *census )
     counter *ctr;
     int count;
 
-    fprintf(hp_file, "BEGIN_SAMPLE %0.2f\n", census->time);
+    printSample(rtsTrue, census->time);
 
 #ifdef PROFILING
     if (RtsFlags.ProfFlags.doHeapProfile == HEAP_BY_LDV) {
@@ -743,7 +754,7 @@ dumpCensus( Census *census )
 		census->prim * sizeof(W_));
 	fprintf(hp_file, "DRAG\t%u\n", census->drag_total *
 		sizeof(W_));
-	fprintf(hp_file, "END_SAMPLE %0.2f\n", census->time);
+	printSample(rtsFalse, census->time);
 	return;
     }
 #endif
@@ -823,7 +834,7 @@ dumpCensus( Census *census )
 	fprintf(hp_file, "\t%d\n", count * sizeof(W_));
     }
 
-    fprintf(hp_file, "END_SAMPLE %0.2f\n", census->time);
+    printSample(rtsFalse, census->time);
 }
 
 /* -----------------------------------------------------------------------------
