@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: Profiling.h,v 1.2 2000/04/19 12:42:48 simonmar Exp $
+ * $Id: Profiling.h,v 1.5 2002/07/18 09:12:03 simonmar Exp $
  *
  * (c) The GHC Team, 1998-2000
  *
@@ -19,15 +19,12 @@ extern FILE *hp_file;
 #ifdef PROFILING
 
 void gen_XML_logfile     ( void );
-void report_ccs_profiling ( void );
-void heap_profile_finish (void);
+void reportCCSProfiling ( void );
 
 void PrintNewStackDecls ( void );
 
-void print_ccs (FILE *, CostCentreStack *);
+extern lnat total_prof_ticks;
 
-# define DEFAULT_INTERVAL TICK_FREQUENCY
-
-extern rtsBool time_profiling;
+extern void fprintCCS( FILE *f, CostCentreStack *ccs );
 
 #endif
