@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: Schedule.h,v 1.22.6.1 2001/12/13 14:32:02 simonmar Exp $
+ * $Id: Schedule.h,v 1.38.6.6 2004/03/02 11:23:17 simonmar Exp $
  *
  * (c) The GHC Team 1998-1999
  *
@@ -8,31 +8,18 @@
  *
  * -------------------------------------------------------------------------*/
 
-//@menu
-//* Scheduler Functions::	
-//* Scheduler Vars and Data Types::  
-//* Some convenient macros::	
-//* Index::			
-//@end menu
+#ifndef __SCHEDULE_H__
+#define __SCHEDULE_H__
+#include "OSThreads.h"
 
-//@node Scheduler Functions, Scheduler Vars and Data Types
-//@subsection Scheduler Functions
-
-//@cindex initScheduler
-//@cindex exitScheduler
-//@cindex startTasks
 /* initScheduler(), exitScheduler(), startTasks()
  * 
  * Called from STG :  no
  * Locks assumed   :  none
  */
-void initScheduler( void );
-void exitScheduler( void );
-#ifdef SMP
-void startTasks( void );
-#endif
+extern void initScheduler  ( void );
+extern void exitScheduler  ( void );
 
-//@cindex awakenBlockedQueue
 /* awakenBlockedQueue()
  *
  * Takes a pointer to the beginning of a blocked TSO queue, and
@@ -46,10 +33,10 @@ void awakenBlockedQueue(StgBlockingQueueElement *q, StgClosure *node);
 #elif defined(PAR)
 void awakenBlockedQueue(StgBlockingQueueElement *q, StgClosure *node);
 #else
-void awakenBlockedQueue(StgTSO *tso);
+void awakenBlockedQueue (StgTSO *tso);
+void awakenBlockedQueueNoLock (StgTSO *tso);
 #endif
 
-//@cindex unblockOne
 /* unblockOne()
  *
  * Takes a pointer to the beginning of a blocked TSO queue, and
@@ -64,7 +51,6 @@ StgBlockingQueueElement *unblockOne(StgBlockingQueueElement *bqe, StgClosure *no
 StgTSO *unblockOne(StgTSO *tso);
 #endif
 
-//@cindex raiseAsync
 /* raiseAsync()
  *
  * Raises an exception asynchronously in the specified thread.
@@ -73,11 +59,11 @@ StgTSO *unblockOne(StgTSO *tso);
  * Locks assumed   :  none
  */
 void raiseAsync(StgTSO *tso, StgClosure *exception);
+void raiseAsyncWithLock(StgTSO *tso, StgClosure *exception);
 
-//@cindex awaitEvent
-/* awaitEvent()
+/* awaitEvent(rtsBool wait)
  *
- * Raises an exception asynchronously in the specified thread.
+ * Checks for blocked threads that need to be woken.
  *
  * Called from STG :  NO
  * Locks assumed   :  sched_mutex
@@ -93,8 +79,34 @@ void awaitEvent(rtsBool wait);  /* In Select.c */
  */
 rtsBool wakeUpSleepingThreads(nat);  /* In Select.c */
 
+/* wakeBlockedWorkerThread()
+ *
+ * If a worker thread is currently blocked in awaitEvent(), interrupt it.
+ *
+ * Called from STG :  NO
+ * Locks assumed   :  sched_mutex
+ */
+void wakeBlockedWorkerThread(void); /* In Select.c */
+
+/* resetWorkerWakeupPipeAfterFork()
+ *
+ * Notify Select.c that a fork() has occured
+ *
+ * Called from STG :  NO
+ * Locks assumed   :  don't care, but must be called right after fork()
+ */
+void resetWorkerWakeupPipeAfterFork(void); /* In Select.c */
+
+/* GetRoots(evac_fn f)
+ *
+ * Call f() for each root known to the scheduler.
+ *
+ * Called from STG :  NO
+ * Locks assumed   :  ????
+ */
+void GetRoots(evac_fn);
+
 // ToDo: check whether all fcts below are used in the SMP version, too
-//@cindex awaken_blocked_queue
 #if defined(GRAN)
 void    awaken_blocked_queue(StgBlockingQueueElement *q, StgClosure *node);
 void    unlink_from_bq(StgTSO* tso, StgClosure* node);
@@ -110,10 +122,6 @@ void    awaken_blocked_queue(StgTSO *q);
 void    initThread(StgTSO *tso, nat stack_size);
 #endif
 
-//@node Scheduler Vars and Data Types, Some convenient macros, Scheduler Functions
-//@subsection Scheduler Vars and Data Types
-
-//@cindex context_switch
 /* Context switch flag.
  * Locks required  : sched_mutex
  */
@@ -122,17 +130,6 @@ extern rtsBool interrupted;
 
 /* In Select.c */
 extern nat timestamp;
-extern nat ticks_since_timestamp;
-
-/* Free capability list.
- * Locks required: sched_mutex.
- */
-#ifdef SMP
-extern Capability *free_capabilities;
-extern nat n_free_capabilities;
-#else
-extern Capability MainCapability;
-#endif
 
 /* Thread queues.
  * Locks required  : sched_mutex
@@ -149,31 +146,22 @@ extern  StgTSO *sleeping_queue;
 /* Linked list of all threads. */
 extern  StgTSO *all_threads;
 
-#ifdef SMP
-//@cindex sched_mutex
-//@cindex thread_ready_cond
-//@cindex gc_pending_cond
-extern pthread_mutex_t sched_mutex;
-extern pthread_cond_t  thread_ready_cond;
-extern pthread_cond_t  gc_pending_cond;
+#if defined(RTS_SUPPORTS_THREADS)
+/* Schedule.c has detailed info on what these do */
+extern Mutex       sched_mutex;
+extern Condition   returning_worker_cond;
+extern nat         rts_n_waiting_workers;
+extern nat         rts_n_waiting_tasks;
 #endif
 
-//@cindex task_info
-#ifdef SMP
-typedef struct {
-  pthread_t id;
-  double    elapsedtimestart;
-  double    mut_time;
-  double    mut_etime;
-  double    gc_time;
-  double    gc_etime;
-} task_info;
+StgBool rtsSupportsBoundThreads(void);
+StgBool isThreadBound(StgTSO *tso);
+StgInt forkProcess(HsStablePtr *entry);
 
-extern task_info *task_ids;
-#endif
+extern SchedulerStatus rts_mainLazyIO(HaskellObj p, /*out*/HaskellObj *ret);
 
-/* Needed by Hugs.
- */
+
+/* Called by shutdown_handler(). */
 void interruptStgRts ( void );
 
 void raiseAsync(StgTSO *tso, StgClosure *exception);
@@ -181,28 +169,65 @@ nat  run_queue_len(void);
 
 void resurrectThreads( StgTSO * );
 
-//@node Some convenient macros, Index, Scheduler Vars and Data Types
-//@subsection Some convenient macros
+/* Main threads:
+ *
+ * These are the threads which clients have requested that we run.  
+ *
+ * In a 'threaded' build, each of these corresponds to one bound thread.
+ * The pointer to the StgMainThread is passed as a parameter to schedule;
+ * this invocation of schedule will always pass this main thread's
+ * bound_thread_cond to waitForkWorkCapability; OS-thread-switching
+ * takes place using passCapability.
+ *
+ * In non-threaded builds, clients are strictly nested: the first client calls
+ * into the RTS, which might call out again to C with a _ccall_GC, and
+ * eventually re-enter the RTS.
+ *
+ * This is non-abstract at the moment because the garbage collector
+ * treats pointers to TSOs from the main thread list as "weak" - these
+ * pointers won't prevent a thread from receiving a BlockedOnDeadMVar
+ * exception.
+ *
+ * Main threads information is kept in a linked list:
+ */
+typedef struct StgMainThread_ {
+  StgTSO *         tso;
+  SchedulerStatus  stat;
+  StgClosure **    ret;
+#if defined(RTS_SUPPORTS_THREADS)
+  Condition        bound_thread_cond;
+#endif
+  struct StgMainThread_ *prev;
+  struct StgMainThread_ *link;
+} StgMainThread;
 
+/* Main thread queue.
+ * Locks required: sched_mutex.
+ */
+extern StgMainThread *main_threads;
+
+void printAllThreads(void);
+#ifdef COMPILING_SCHEDULER
+static void printThreadBlockage(StgTSO *tso);
+static void printThreadStatus(StgTSO *tso);
+#endif
 /* debugging only 
  */
 #ifdef DEBUG
-void printThreadBlockage(StgTSO *tso);
-void printThreadStatus(StgTSO *tso);
-void printAllThreads(void);
-#endif
 void print_bq (StgClosure *node);
+#endif
 #if defined(PAR)
 void print_bqe (StgBlockingQueueElement *bqe);
 #endif
 
+void labelThread(StgPtr tso, char *label);
+
 /* -----------------------------------------------------------------------------
  * Some convenient macros...
  */
 
 /* END_TSO_QUEUE and friends now defined in includes/StgMiscClosures.h */
 
-//@cindex APPEND_TO_RUN_QUEUE
 /* Add a thread to the end of the run queue.
  * NOTE: tso->link should be END_TSO_QUEUE before calling this macro.
  */
@@ -215,7 +240,6 @@ void print_bqe (StgBlockingQueueElement *bqe);
     }						\
     run_queue_tl = tso;
 
-//@cindex PUSH_ON_RUN_QUEUE
 /* Push a thread on the beginning of the run queue.  Used for
  * newly awakened threads, so they get run as soon as possible.
  */
@@ -226,7 +250,6 @@ void print_bqe (StgBlockingQueueElement *bqe);
       run_queue_tl = tso;			\
     }
 
-//@cindex POP_RUN_QUEUE
 /* Pop the first thread off the runnable queue.
  */
 #define POP_RUN_QUEUE()				\
@@ -241,7 +264,6 @@ void print_bqe (StgBlockingQueueElement *bqe);
     t;						\
   })
 
-//@cindex APPEND_TO_BLOCKED_QUEUE
 /* Add a thread to the end of the blocked queue.
  */
 #define APPEND_TO_BLOCKED_QUEUE(tso)		\
@@ -253,46 +275,41 @@ void print_bqe (StgBlockingQueueElement *bqe);
     }						\
     blocked_queue_tl = tso;
 
-//@cindex THREAD_RUNNABLE
 /* Signal that a runnable thread has become available, in
  * case there are any waiting tasks to execute it.
  */
-#ifdef SMP
+#if defined(RTS_SUPPORTS_THREADS)
 #define THREAD_RUNNABLE()			\
-  if (free_capabilities != NULL) {		\
-     pthread_cond_signal(&thread_ready_cond);	\
-  }						\
+  wakeBlockedWorkerThread();			\
   context_switch = 1;
 #else
 #define THREAD_RUNNABLE()  /* nothing */
 #endif
 
-//@cindex EMPTY_RUN_QUEUE
-/* Check whether the run queue is empty i.e. the PE is idle
+/* Check whether various thread queues are empty
  */
-#define EMPTY_RUN_QUEUE()     (run_queue_hd == END_TSO_QUEUE)
+#define EMPTY_QUEUE(q)         (q == END_TSO_QUEUE)
 
-//@node Index,  , Some convenient macros
-//@subsection Index
+#define EMPTY_RUN_QUEUE()      (EMPTY_QUEUE(run_queue_hd))
+#define EMPTY_BLOCKED_QUEUE()  (EMPTY_QUEUE(blocked_queue_hd))
+#define EMPTY_SLEEPING_QUEUE() (EMPTY_QUEUE(sleeping_queue))
 
-//@index
-//* APPEND_TO_BLOCKED_QUEUE::  @cindex\s-+APPEND_TO_BLOCKED_QUEUE
-//* APPEND_TO_RUN_QUEUE::  @cindex\s-+APPEND_TO_RUN_QUEUE
-//* Capability::  @cindex\s-+Capability
-//* POP_RUN_QUEUE    ::  @cindex\s-+POP_RUN_QUEUE    
-//* PUSH_ON_RUN_QUEUE::  @cindex\s-+PUSH_ON_RUN_QUEUE
-//* THREAD_RUNNABLE::  @cindex\s-+THREAD_RUNNABLE
-//* awaitEvent::  @cindex\s-+awaitEvent
-//* awakenBlockedQueue::  @cindex\s-+awakenBlockedQueue
-//* awaken_blocked_queue::  @cindex\s-+awaken_blocked_queue
-//* context_switch::  @cindex\s-+context_switch
-//* exitScheduler::  @cindex\s-+exitScheduler
-//* gc_pending_cond::  @cindex\s-+gc_pending_cond
-//* initScheduler::  @cindex\s-+initScheduler
-//* raiseAsync::  @cindex\s-+raiseAsync
-//* sched_mutex::  @cindex\s-+sched_mutex
-//* startTasks::  @cindex\s-+startTasks
-//* task_info::  @cindex\s-+task_info
-//* thread_ready_cond::  @cindex\s-+thread_ready_cond
-//* unblockOne::  @cindex\s-+unblockOne
-//@end index
+#define EMPTY_THREAD_QUEUES()  (EMPTY_RUN_QUEUE() && \
+				EMPTY_BLOCKED_QUEUE() && \
+				EMPTY_SLEEPING_QUEUE())
+
+#if defined(RTS_SUPPORTS_THREADS)
+/* If no task is waiting for a capability,
+ * and if there is work to be done
+ * or if we need to wait for IO or delay requests,
+ * spawn a new worker thread.
+ */
+void
+startSchedulerTaskIfNecessary(void);
+#endif
+
+#ifdef DEBUG
+extern void sched_belch(char *s, ...);
+#endif
+
+#endif /* __SCHEDULE_H__ */
