@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: Stats.c,v 1.32.4.3 2001/12/13 14:38:22 simonmar Exp $
+ * $Id: Stats.c,v 1.46.12.1 2004/05/27 09:32:15 simonmar Exp $
  *
  * (c) The GHC Team, 1998-1999
  *
@@ -18,7 +18,8 @@
 #include "MBlock.h"
 #include "Schedule.h"
 #include "Stats.h"
-#include "ParTicky.h"                       // ToDo: move into Rts.h
+#include "ParTicky.h"                       /* ToDo: move into Rts.h */
+#include "Profiling.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -93,12 +94,26 @@ static TICK_TYPE ExitElapsedTime  = 0;
 static ullong GC_tot_alloc        = 0;
 static ullong GC_tot_copied       = 0;
 
-static TICK_TYPE GC_start_time,  GC_tot_time = 0;  /* User GC Time */
-static TICK_TYPE GCe_start_time, GCe_tot_time = 0; /* Elapsed GC time */
+static TICK_TYPE GC_start_time = 0,  GC_tot_time  = 0;  /* User GC Time */
+static TICK_TYPE GCe_start_time = 0, GCe_tot_time = 0;  /* Elapsed GC time */
 
-lnat MaxResidency = 0;     /* in words; for stats only */
-lnat AvgResidency = 0;
-lnat ResidencySamples = 0; /* for stats only */
+#ifdef PROFILING
+static TICK_TYPE RP_start_time  = 0, RP_tot_time  = 0;  /* retainer prof user time */
+static TICK_TYPE RPe_start_time = 0, RPe_tot_time = 0;  /* retainer prof elap time */
+
+static TICK_TYPE HC_start_time, HC_tot_time = 0;     // heap census prof user time
+static TICK_TYPE HCe_start_time, HCe_tot_time = 0;   // heap census prof elap time
+#endif
+
+#ifdef PROFILING
+#define PROF_VAL(x)   (x)
+#else
+#define PROF_VAL(x)   0
+#endif
+
+static lnat MaxResidency = 0;     // in words; for stats only
+static lnat AvgResidency = 0;
+static lnat ResidencySamples = 0; // for stats only
 
 static lnat GC_start_faults = 0, GC_end_faults = 0;
 
@@ -210,18 +225,37 @@ getTimes(void)
  *	      stat_startGC() for details)
  */
 double
-mut_user_time_during_GC(void)
+mut_user_time_during_GC( void )
 {
-  return TICK_TO_DBL(GC_start_time - GC_tot_time);
+  return TICK_TO_DBL(GC_start_time - GC_tot_time - PROF_VAL(RP_tot_time + HC_tot_time));
 }
 
 double
-mut_user_time(void)
+mut_user_time( void )
 {
     getTimes();
-    return TICK_TO_DBL(CurrentUserTime - GC_tot_time);
+    return TICK_TO_DBL(CurrentUserTime - GC_tot_time - PROF_VAL(RP_tot_time + HC_tot_time));
 }
 
+#ifdef PROFILING
+/*
+  mut_user_time_during_RP() is similar to mut_user_time_during_GC();
+  it returns the MUT time during retainer profiling.
+  The same is for mut_user_time_during_HC();
+ */
+double
+mut_user_time_during_RP( void )
+{
+  return TICK_TO_DBL(RP_start_time - GC_tot_time - RP_tot_time - HC_tot_time);
+}
+
+double
+mut_user_time_during_heap_census( void )
+{
+  return TICK_TO_DBL(HC_start_time - GC_tot_time - RP_tot_time - HC_tot_time);
+}
+#endif /* PROFILING */
+
 static nat
 pageFaults(void)
 {
@@ -320,9 +354,10 @@ stat_startExit(void)
 {
     getTimes();
     MutElapsedStamp = CurrentElapsedTime;
-    MutElapsedTime = CurrentElapsedTime - GCe_tot_time - InitElapsedStamp;
+    MutElapsedTime = CurrentElapsedTime - GCe_tot_time -
+	PROF_VAL(RPe_tot_time + HCe_tot_time) - InitElapsedStamp;
     if (MutElapsedTime < 0) { MutElapsedTime = 0; }	/* sometimes -0.00 */
-    
+
     /* for SMP, we don't know the mutator time yet, we have to inspect
      * all the running threads to find out, and they haven't stopped
      * yet.  So we just timestamp MutUserTime at this point so we can
@@ -332,7 +367,7 @@ stat_startExit(void)
 #ifdef SMP
     MutUserTime = CurrentUserTime;
 #else
-    MutUserTime = CurrentUserTime - GC_tot_time - InitUserTime;
+    MutUserTime = CurrentUserTime - GC_tot_time - PROF_VAL(RP_tot_time + HC_tot_time) - InitUserTime;
     if (MutUserTime < 0) { MutUserTime = 0; }
 #endif
 }
@@ -344,7 +379,7 @@ stat_endExit(void)
 #ifdef SMP
     ExitUserTime = CurrentUserTime - MutUserTime;
 #else
-    ExitUserTime = CurrentUserTime - MutUserTime - GC_tot_time - InitUserTime;
+    ExitUserTime = CurrentUserTime - MutUserTime - GC_tot_time - PROF_VAL(RP_tot_time + HC_tot_time) - InitUserTime;
 #endif
     ExitElapsedTime = CurrentElapsedTime - MutElapsedStamp;
     if (ExitUserTime < 0) {
@@ -470,6 +505,73 @@ stat_endGC(lnat alloc, lnat collect, lnat live, lnat copied, lnat gen)
     }
 }
 
+/* -----------------------------------------------------------------------------
+   Called at the beginning of each Retainer Profiliing
+   -------------------------------------------------------------------------- */
+#ifdef PROFILING
+void
+stat_startRP(void)
+{
+  getTimes();
+  RP_start_time = CurrentUserTime;
+  RPe_start_time = CurrentElapsedTime;
+}
+#endif /* PROFILING */
+
+/* -----------------------------------------------------------------------------
+   Called at the end of each Retainer Profiliing
+   -------------------------------------------------------------------------- */
+
+#ifdef PROFILING
+void
+stat_endRP(
+  nat retainerGeneration,
+#ifdef DEBUG_RETAINER
+  nat maxCStackSize,
+  int maxStackSize,
+#endif
+  double averageNumVisit)
+{
+  getTimes();
+  RP_tot_time += CurrentUserTime - RP_start_time;
+  RPe_tot_time += CurrentElapsedTime - RPe_start_time;
+
+  fprintf(prof_file, "Retainer Profiling: %d, at %f seconds\n", 
+    retainerGeneration, mut_user_time_during_RP());
+#ifdef DEBUG_RETAINER
+  fprintf(prof_file, "\tMax C stack size = %u\n", maxCStackSize);
+  fprintf(prof_file, "\tMax auxiliary stack size = %u\n", maxStackSize);
+#endif
+  fprintf(prof_file, "\tAverage number of visits per object = %f\n", averageNumVisit);
+}
+#endif /* PROFILING */
+
+/* -----------------------------------------------------------------------------
+   Called at the beginning of each heap census
+   -------------------------------------------------------------------------- */
+#ifdef PROFILING
+void
+stat_startHeapCensus(void)
+{
+  getTimes();
+  HC_start_time = CurrentUserTime;
+  HCe_start_time = CurrentElapsedTime;
+}
+#endif /* PROFILING */
+
+/* -----------------------------------------------------------------------------
+   Called at the end of each heap census
+   -------------------------------------------------------------------------- */
+#ifdef PROFILING
+void
+stat_endHeapCensus(void) 
+{
+  getTimes();
+  HC_tot_time += CurrentUserTime - HC_start_time;
+  HCe_tot_time += CurrentElapsedTime - HCe_start_time;
+}
+#endif /* PROFILING */
+
 /* -----------------------------------------------------------------------------
    stat_workerStop
 
@@ -477,17 +579,19 @@ stat_endGC(lnat alloc, lnat collect, lnat live, lnat copied, lnat gen)
    stats for this thread into the task_ids struct for that thread.
    -------------------------------------------------------------------------- */
 
-#ifdef SMP
+#if defined(SMP)
 void
 stat_workerStop(void)
 {
     nat i;
     pthread_t me = pthread_self();
 
+    getTimes();
+
     for (i = 0; i < RtsFlags.ParFlags.nNodes; i++) {
 	if (task_ids[i].id == me) {
-	    task_ids[i].mut_time = usertime() - task_ids[i].gc_time;
-	    task_ids[i].mut_etime = elapsedtime()
+	    task_ids[i].mut_time = CurrentUserTime - task_ids[i].gc_time;
+	    task_ids[i].mut_etime = CurrentElapsedTime
 		- GCe_tot_time
 		- task_ids[i].elapsedtimestart;
 	    if (task_ids[i].mut_time < 0.0)  { task_ids[i].mut_time = 0.0;  }
@@ -497,6 +601,14 @@ stat_workerStop(void)
 }
 #endif
 
+#if defined(SMP)
+long int stat_getElapsedTime ()
+{
+  getTimes();
+  return CurrentElapsedTime;
+}
+#endif
+
 /* -----------------------------------------------------------------------------
    Called at the end of execution
 
@@ -598,6 +710,12 @@ stat_exit(int alloc)
 		    TICK_TO_DBL(MutUserTime), TICK_TO_DBL(MutElapsedTime));
 	    fprintf(sf, "  GC    time  %6.2fs  (%6.2fs elapsed)\n",
 		    TICK_TO_DBL(GC_tot_time), TICK_TO_DBL(GCe_tot_time));
+#ifdef PROFILING
+	    fprintf(sf, "  RP    time  %6.2fs  (%6.2fs elapsed)\n",
+		    TICK_TO_DBL(RP_tot_time), TICK_TO_DBL(RPe_tot_time));
+	    fprintf(sf, "  PROF  time  %6.2fs  (%6.2fs elapsed)\n",
+		    TICK_TO_DBL(HC_tot_time), TICK_TO_DBL(HCe_tot_time));
+#endif 
 	    fprintf(sf, "  EXIT  time  %6.2fs  (%6.2fs elapsed)\n",
 		    TICK_TO_DBL(ExitUserTime), TICK_TO_DBL(ExitElapsedTime));
 	    fprintf(sf, "  Total time  %6.2fs  (%6.2fs elapsed)\n\n",
@@ -606,20 +724,23 @@ stat_exit(int alloc)
 		    TICK_TO_DBL(GC_tot_time)*100/TICK_TO_DBL(time),
 		    TICK_TO_DBL(GCe_tot_time)*100/TICK_TO_DBL(etime));
 
-	    if (time - GC_tot_time == 0)
+	    if (time - GC_tot_time - PROF_VAL(RP_tot_time + HC_tot_time) == 0)
 		ullong_format_string(0, temp, rtsTrue/*commas*/);
 	    else
 		ullong_format_string(
 		    (ullong)((GC_tot_alloc*sizeof(W_))/
-			     TICK_TO_DBL(time - GC_tot_time)),
+			     TICK_TO_DBL(time - GC_tot_time - 
+					 PROF_VAL(RP_tot_time + HC_tot_time))),
 		    temp, rtsTrue/*commas*/);
 	    
 	    fprintf(sf, "  Alloc rate    %s bytes per MUT second\n\n", temp);
 	
 	    fprintf(sf, "  Productivity %5.1f%% of total user, %.1f%% of total elapsed\n\n",
-		    TICK_TO_DBL(time - GC_tot_time - InitUserTime) * 100 
+		    TICK_TO_DBL(time - GC_tot_time - 
+				PROF_VAL(RP_tot_time + HC_tot_time) - InitUserTime) * 100 
 		    / TICK_TO_DBL(time), 
-		    TICK_TO_DBL(time - GC_tot_time - InitUserTime) * 100 
+		    TICK_TO_DBL(time - GC_tot_time - 
+				PROF_VAL(RP_tot_time + HC_tot_time) - InitUserTime) * 100 
 		    / TICK_TO_DBL(etime));
 	}
 
@@ -628,7 +749,8 @@ stat_exit(int alloc)
 	  fprintf(sf, "<<ghc: %llu bytes, ", GC_tot_alloc*sizeof(W_));
 	  fprintf(sf, "%d GCs, %ld/%ld avg/max bytes residency (%ld samples), %luM in use, %.2f INIT (%.2f elapsed), %.2f MUT (%.2f elapsed), %.2f GC (%.2f elapsed) :ghc>>\n",
 		    total_collections,
-		    AvgResidency*sizeof(W_)/ResidencySamples, 
+		    ResidencySamples == 0 ? 0 : 
+		        AvgResidency*sizeof(W_)/ResidencySamples, 
 		    MaxResidency*sizeof(W_), 
 		    ResidencySamples,
 		    (unsigned long)(mblocks_allocated * MBLOCK_SIZE / (1024L * 1024L)),
@@ -649,6 +771,7 @@ stat_exit(int alloc)
 
    Produce some detailed info on the state of the generational GC.
    -------------------------------------------------------------------------- */
+#ifdef DEBUG
 void
 statDescribeGens(void)
 {
@@ -691,11 +814,12 @@ statDescribeGens(void)
   }
   fprintf(stderr,"\n");
 }
+#endif
 
 /* -----------------------------------------------------------------------------
    Stats available via a programmatic interface, so eg. GHCi can time
    each compilation and expression evaluation.
    -------------------------------------------------------------------------- */
 
-extern HsInt getAllocations( void ) 
-{ return (HsInt)(total_allocated * sizeof(W_)); }
+extern HsInt64 getAllocations( void ) 
+{ return (HsInt64)total_allocated * sizeof(W_); }
