@@ -1,5 +1,5 @@
 /* -----------------------------------------------------------------------------
- * $Id: Storage.c,v 1.79.6.2 2003/11/10 10:26:49 simonmar Exp $
+ * $Id: Storage.c,v 1.79.6.3 2004/07/21 10:48:56 simonmar Exp $
  *
  * (c) The GHC Team, 1998-1999
  *
@@ -44,7 +44,7 @@ generation *g0		= NULL; /* generation 0, for convenience */
 generation *oldest_gen  = NULL; /* oldest generation, for convenience */
 step *g0s0 		= NULL; /* generation 0, step 0, for convenience */
 
-lnat total_allocated = 0;	/* total memory allocated during run */
+ullong total_allocated = 0;	/* total memory allocated during run */
 
 /*
  * Storage manager mutex:  protects all the above state from
