@@ -7,6 +7,7 @@
 module Main where
 
 import Package
+import Version	 ( version )
 
 #if __GLASGOW_HASKELL__ >= 504
 import System.Console.GetOpt
@@ -82,11 +83,6 @@ isAction _              = True
 copyright :: String
 copyright = "GHC package manager version " ++ version ++ "\n"
 
--- hackery to convice cpp to splice GHC_PKG_VERSION into a string
-version :: String
-version = tail "\ 
-   \ GHC_PKG_VERSION"
-
 usageHeader :: String -> String
 usageHeader prog = "Usage: " ++ prog ++ " [OPTION...]\n"
 
@@ -256,9 +252,9 @@ checkConfigAccess filename = do
 maybeRestoreOldConfig :: FilePath -> IO () -> IO ()
 maybeRestoreOldConfig filename io
   = my_catch io (\e -> do
-        hPutStr stdout "\nWARNING: an error was encountered while the new \n\ 
-        	       \configuration was being written.  Attempting to \n\ 
-        	       \restore the old configuration... "
+        hPutStr stdout ("\nWARNING: an error was encountered while the new \n"++
+        	          "configuration was being written.  Attempting to \n"++
+        	          "restore the old configuration... ")
 	renameFile (filename ++ ".old")  filename
         hPutStrLn stdout "done."
 	my_throw e
@@ -371,9 +367,15 @@ autoBuildGHCiLib dir batch_file ghci_file = do
 #ifdef darwin_TARGET_OS
   system("ld -r -x -o " ++ ghci_lib_file ++ 
 	 " -all_load " ++ batch_lib_file)
+#else
+#ifdef mingw32_HOST_OS
+  execDir <- getExecDir "/bin/ghc-pkg.exe"
+  system (maybe "" (++"/gcc-lib/") execDir++"ld -r -x -o " ++ ghci_lib_file ++ 
+	 " --whole-archive " ++ batch_lib_file)
 #else
   system("ld -r -x -o " ++ ghci_lib_file ++ 
 	 " --whole-archive " ++ batch_lib_file)
+#endif
 #endif
   hPutStrLn stderr (" done.")
 
@@ -524,7 +526,7 @@ getExecDir cmd
   where
     len = 2048::Int -- Plenty, PATH_MAX is 512 under Win32.
 
-foreign import stdcall "GetModuleFileNameA" unsafe 
+foreign import stdcall unsafe  "GetModuleFileNameA"
   getModuleFileName :: Ptr () -> CString -> Int -> IO Int32
 #else
 getExecDir :: String -> IO (Maybe String) 
