@@ -73,13 +73,13 @@ CommaPrint(fp,n)
         fprintf(fp, "%d", (int)n);
     } else {
         CommaPrint(fp, n / ONETHOUSAND);
-        fprintf(fp, ",%03d", (int)n % ONETHOUSAND);
+        fprintf(fp, ",%03d", (int)(n % ONETHOUSAND));
     }
 }
 
 void *
 xmalloc(n)
-  int n;
+  size_t n;
 {
     void *r;
 
@@ -94,7 +94,7 @@ xmalloc(n)
 void *
 xrealloc(p, n)
   void *p;
-  int n;
+  size_t n;
 {
     void *r;
     extern void *realloc();
