@@ -1,5 +1,5 @@
 module GraphicsTime
-	( Time(..)
+	( Time
 	, getTime
 	) where
 
