@@ -3,4 +3,4 @@
 
 /*	Install the cross call procedures in the gCrossCallProcedureTable of cCrossCall_121.
 */
-extern int InstallCrossCallPrinter (int ios);
+extern void InstallCrossCallPrinter ();
