@@ -1,6 +1,6 @@
 /********************************************************************************************
 	Clean OS Windows library module version 1.2.1.
-	This module is part of the Clean Object I/O library, version 1.2.1, 
+	This module is part of the Clean Object I/O library, version 1.2.1,
 	for the Windows platform.
 ********************************************************************************************/
 
@@ -9,12 +9,12 @@
 	cCrossCall_121 defines the infrastructure required by the Object I/O library to call
 	system procedures that interact with the Windows callback mechanism.
 
-	The basic principle in cCrossCall_121 is to have a minimal cross call kernel. If Clean 
-	code requires extension of the functionality of the OS thread, then this functionality 
-	must be	registered before being applicable. 
+	The basic principle in cCrossCall_121 is to have a minimal cross call kernel. If Clean
+	code requires extension of the functionality of the OS thread, then this functionality
+	must be	registered before being applicable.
 
-	In this version the request codes are still statically fixed and are assumed to be 
-	globally available both in the OS thread and the Clean thread. In a future version this 
+	In this version the request codes are still statically fixed and are assumed to be
+	globally available both in the OS thread and the Clean thread. In a future version this
 	will probably be replaced by a dynamic allocation of cross call request codes.
 ********************************************************************************************/
 
@@ -52,7 +52,7 @@ CrossCallProcedureTable gCrossCallProcedureTable;
 /**********************************************************************************************
 	Internal global data section.
 **********************************************************************************************/
-static BOOL gEventsInited        = FALSE;
+
 static BOOL gAppRunning          = FALSE;
 static HANDLE gCLEAN_DONE;
 static HANDLE gOS_DONE;
@@ -70,7 +70,7 @@ static CrossCallInfo *MakeQuitCci (CrossCallInfo * pcci);
 /*	Menu(item)IDs are not allowed to exceed OSMenuIDEnd.
 	This is because window ids start at (OSMenuIDEnd+5), and need to be distinct from menu ids
 	in case of MDI processes.
-	The global gMenuItemID (initially 0) is incremented by NextMenuItemID each time a new 
+	The global gMenuItemID (initially 0) is incremented by NextMenuItemID each time a new
 	menu(item)ID is required.
 	This implementation does not reuse freed ids and is therefore not adequate!!
 */
@@ -197,12 +197,12 @@ int CheckVirtualKeyCode (int keycode)
 }
 
 
-/*	EndSuspendTimerProc is the parameter of the SetTimer procedure used 
+/*	EndSuspendTimerProc is the parameter of the SetTimer procedure used
 	in GetMessageQuickly to set a positive timer interval.
 	The WaitMessage routine suspends the OS thread until an interesting
 	event has occurred or the timer has been triggered. In that case
 	EndSuspendTimerProc is called, which kills the timer and informs
-	Clean about the timer event. 
+	Clean about the timer event.
 */
 static VOID CALLBACK EndSuspendTimerProc (HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime )
 {
@@ -242,10 +242,10 @@ static int GetMessageQuickly (BOOL gIdleTimerOn, int gSleeptime, MSG * pmsg)
 		else
 		{
 			POINT p;
-			
+
 			/*	The following code has been inserted to reduce the crosscall traffic.
 				A timer is set to suspend this thread until atleast the timer interval
-				has elapsed. 
+				has elapsed.
 			*/
 			if (SetTimer (ghMainWindow, (UINT) -2, (UINT)gSleeptime, &EndSuspendTimerProc))
 			{
@@ -273,7 +273,7 @@ static int GetMessageQuickly (BOOL gIdleTimerOn, int gSleeptime, MSG * pmsg)
 
 
 
-//	TranslateKeyboardMessage is used by HandleCleanRequest only. 
+//	TranslateKeyboardMessage is used by HandleCleanRequest only.
 static BOOL TranslateKeyboardMessage (MSG * pmsg)
 {
 	int c;
@@ -306,7 +306,7 @@ void HandleCleanRequest (CrossCallInfo * pcci)
 {
 	switch (pcci->mess)
 	{
-		case CcRqDOMESSAGE: 	// idleTimerOn, sleeptime; no result. 
+		case CcRqDOMESSAGE: 	// idleTimerOn, sleeptime; no result.
 			{
 				MSG ms;
 				int msgresult;
@@ -331,10 +331,10 @@ void HandleCleanRequest (CrossCallInfo * pcci)
 							{	// Verify the correctness of gAcceleratorTable
 								gAcceleratorTable = UpdateAcceleratorTable (gAcceleratorTable,ghActiveFrameWindow);
 							}
-						
+
 							if (gAcceleratorTable==NULL || !TranslateAccelerator (ghActiveFrameWindow, gAcceleratorTable, &ms))
 							{
-								{	
+								{
 									TranslateKeyboardMessage (&ms);
 									DispatchMessage (&ms);
 								}
@@ -348,7 +348,7 @@ void HandleCleanRequest (CrossCallInfo * pcci)
 		default:
 			{
 				CrossCallProcedure action;
-				
+
 				action = FindCrossCallEntry (gCrossCallProcedureTable, pcci->mess);
 
 				if (action == NULL)
@@ -375,44 +375,12 @@ BOOL OsThreadRunning (void)
 	return GetCurrentThreadId () == gOsThreadId;
 }
 
-void WinInitOs (Bool * ok, OS * os)
-{
-	if (gEventsInited)
-	{
-		*ok = FALSE;
-		rprintf ("WIO: *ok = FALSE\n");
-	}
-	else
-	{
-		*ok = TRUE;
-		gEventsInited = TRUE;
-		rprintf ("WIO: *ok = TRUE\n");
-	}
-
-	*os = 54321;
-}	/* WinInitOs */
-
-Bool WinCloseOs (OS os)
-{
-	if (gEventsInited)
-	{
-		rprintf ("WCO: return TRUE\n");
-		gEventsInited = FALSE;
-		return TRUE;
-	}
-	else
-	{
-		rprintf ("WCO: return FALSE\n");
-		return FALSE;
-	}
-}	/* WinCloseOs */
-
 /*	InitGlobals is used by WinStartOSThread only. */
 static void InitGlobals (void)
 {
 	LOGFONT lf;
-	
-	//	Globally, we create a logical font that is used in all controls. 
+
+	//	Globally, we create a logical font that is used in all controls.
 	SetLogFontData (&lf, "MS Sans Serif", 0, 8);
 	gControlFont = CreateFontIndirect (&lf);
 
@@ -420,7 +388,7 @@ static void InitGlobals (void)
 	gCrossCallProcedureTable = EmptyCrossCallProcedureTable ();
 }	/* InitGlobals */
 
-OS WinStartOsThread (OS os)
+void WinStartOsThread ()
 {
 	rprintf ("WSOT: Started\n");
 
@@ -450,13 +418,11 @@ OS WinStartOsThread (OS os)
 	rprintf ("WSOT: new thread created\n");
 	WaitForSingleObject (gOS_DONE, INFINITE);
 	rprintf ("WSOT: wait done\n");
-
-	return os;
 }	/* WinStartOSThread */
 
-OS WinKillOsThread (OS os)
+void WinKillOsThread ()
 {
-	rprintf ("OS WinKillOsThread (OS os)\n");
+	rprintf ("void WinKillOsThread ()\n");
 	if (ghOSThreadHandle != NULL)
 	{
 		rprintf ("ghOSThreadHandle != NULL\n");
@@ -490,18 +456,14 @@ OS WinKillOsThread (OS os)
 // MW...
 	ghMainWindow = NULL;
 // ... MW
-
-	return os;
 }	/* WinKillOsThread*/
 
 void WinKickOsThread (int imess,
 					  int ip1, int ip2, int ip3,
 					  int ip4, int ip5, int ip6,
-					  OS ios,
 					  int *omess,
 					  int *op1, int *op2, int *op3,
-					  int *op4, int *op5, int *op6,
-					  OS * oos
+					  int *op4, int *op5, int *op6
 					 )
 {
 /*	rprintf("KOT: filling in Cci\n"); */
@@ -531,7 +493,6 @@ void WinKickOsThread (int imess,
 		*op4 = gCci.p4;
 		*op5 = gCci.p5;
 		*op6 = gCci.p6;
-		*oos = ios;
 		rprintf ("KOT: Cci read: {");
 		printCCI (&gCci);
 		rprintf ("}\n");
@@ -548,8 +509,8 @@ void WinKickOsThread (int imess,
 		*op4 = 0;
 		*op5 = 0;
 		*op6 = 0;
-		*oos = ios;
 	}
+
 	rprintf ("KOT: done.\n");
 }	/* WinKickOsThread */
 
@@ -564,7 +525,6 @@ static int clsp = -1;
 
 void KickCleanThread (CrossCallInfo * pcci)
 {
-
 #ifdef PRINTCROSSCALLS
 	if (ossp == -1)
 	{
@@ -733,7 +693,7 @@ BOOL IsReturnCci (CrossCallInfo * pcci)
 extern double c_div_real (double n, double d);
 extern int c_ftoi (double d);
 
-//	GetAppFileName used by OsThreadFunction only. 
+//	GetAppFileName used by OsThreadFunction only.
 static void GetAppFileName (void)
 {
 	char path[MAX_PATH + 1];
@@ -790,8 +750,8 @@ static char MainWindowClassName[] = "__CleanMainWindow";	/* Class for main windo
 
 
 /*	The callback routine for the main window.
-	PA: The WM_CREATE  message registers the main window as a clipboard viewer. 
-		The WM_DESTROY message unregisters the main window. 
+	PA: The WM_CREATE  message registers the main window as a clipboard viewer.
+		The WM_DESTROY message unregisters the main window.
 */
 static LRESULT CALLBACK MainWindowProcedure (HWND hWin, UINT uMess, WPARAM wPara, LPARAM lPara)
 {
@@ -818,7 +778,7 @@ static LRESULT CALLBACK MainWindowProcedure (HWND hWin, UINT uMess, WPARAM wPara
 		case WM_ENTERIDLE:
 			{
 				HWND hwndModalDialog;
-				
+
 				hwndModalDialog = (HWND)lPara;
 
 				if (wPara == MSGF_DIALOGBOX && hwndModalDialog != ghwndLastModalDialog)
@@ -1032,7 +992,7 @@ DWORD OsThreadFunction (DWORD param)
 								(HANDLE) ghInst,				// The instance
 								NULL							// No window creation data
 							 );
-	
+
 	mainSystemMenu = GetSystemMenu (ghMainWindow,FALSE);
 	RemoveMenu (mainSystemMenu, SC_RESTORE,  MF_BYCOMMAND);
 	RemoveMenu (mainSystemMenu, SC_MOVE,     MF_BYCOMMAND);
@@ -1053,480 +1013,3 @@ DWORD OsThreadFunction (DWORD param)
 
 	return 0;
 }	/* OsThreadFunction */
-
-
-PSTR WinGetAppPath (void)
-{
-	char *path;
-	int idx, length;
-
-	path = rmalloc (261);
-
-	length = GetModuleFileName (NULL, path, 260);
-
-	for (idx = length - 1;
-		 path[idx] != '/' &&
-		 path[idx] != '\\' &&
-		 path[idx] != ':';
-		 idx--)
-		;
-
-	path[idx + 1] = 0;
-
-	return path;	/* relying on the calling clean function to de-allocate path. */
-}	/* WinGetAppPath */
-
-Bool WinFileExists (CLEAN_STRING name)
-{
-	char *file_name;
-	HANDLE handle;
-	WIN32_FIND_DATA find_data;
-
-	file_name = cstring (name);
-
-	handle = FindFirstFile (file_name, &find_data);
-
-	if (handle != INVALID_HANDLE_VALUE)
-	{
-		FindClose (handle);
-
-		return TRUE;
-	}
-	else
-		return FALSE;
-}	/* WinFileExists */
-
-void WinCallProcess (PSTR commandline,
-					 PSTR env,
-					 PSTR dir,
-					 PSTR in,
-					 PSTR out,
-					 PSTR err,
-					 OS ios,
-					 Bool * success,
-					 int *exitcode,
-					 OS * oos
-				    )
-{
-	SECURITY_ATTRIBUTES sa;
-	STARTUPINFO si;
-	PROCESS_INFORMATION pi;
-	BOOL fsuccess;
-	char *ep;
-	HANDLE saveStdin, infile;
-	HANDLE saveStdout, outfile;
-	HANDLE saveStderr, errfile;
-
-	rprintf ("WCP: starting...\n");
-
-	rprintf ("success = %d, value = %d\n", success, *success);
-	*success = FALSE;
-	rprintf ("exitcode = %d; value = %d\n", exitcode, *exitcode);
-	*exitcode = -1;
-	rprintf ("oos = %d; value = %d\n", oos, *oos);
-	*oos = ios;
-
-	rprintf ("commandline = %d; value = %s\n", commandline, commandline);
-
-	if (commandline != NULL)
-		rprintf ("WCP: commandline = \"%s\"\n", commandline);
-	else
-		rprintf ("WCP: commandline empty\n");
-
-	if (env != NULL)
-	{
-		ep = env;
-		while (rstrlen (ep) != 0)
-		{
-			rprintf ("WCP:		   env = \"%s\"\n", ep);
-			ep += rstrlen (ep) + 1;
-		}
-	}
-	else
-		rprintf ("WCP:		   env empty\n");
-
-	if (dir != NULL)
-		rprintf ("WCP: dir = \"%s\"\n", dir);
-	else
-		rprintf ("WCP: dir empty\n");
-
-	if (in != NULL)
-		rprintf ("WCP: in = \"%s\"\n", in);
-	else
-		rprintf ("WCP: in empty\n");
-
-	if (out != NULL)
-		rprintf ("WCP: out = \"%s\"\n", out);
-	else
-		rprintf ("WCP: out empty\n");
-
-	if (err != NULL)
-		rprintf ("WCP: err = \"%s\"\n", err);
-	else
-		rprintf ("WCP: err empty\n");
-
-	sa.nLength = sizeof (SECURITY_ATTRIBUTES);
-	sa.bInheritHandle = TRUE;
-	sa.lpSecurityDescriptor = NULL;
-
-	if (in != NULL)
-	{
-		infile = CreateFile (in,
-							 GENERIC_READ,
-							 FILE_SHARE_READ | FILE_SHARE_WRITE,
-							 &sa,
-							 OPEN_EXISTING,
-							 FILE_ATTRIBUTE_NORMAL,
-							 NULL
-			);
-		if (infile == INVALID_HANDLE_VALUE)
-		{
-			rprintf ("infile creation failed\n");
-			return;
-		}
-
-		saveStdin = GetStdHandle (STD_INPUT_HANDLE);
-
-		if (!SetStdHandle (STD_INPUT_HANDLE, infile))
-		{
-			rprintf ("could not redirect input\n");
-			return;
-		}
-		rprintf ("redirection of input ok\n");
-	}
-	else
-	{
-		rprintf ("in == NULL\n");
-		infile = NULL;
-	}
-
-	if (out != NULL)
-	{
-		outfile = CreateFile (out,
-							  GENERIC_WRITE,
-							  0,
-							  &sa,
-							  CREATE_ALWAYS,
-							  FILE_ATTRIBUTE_NORMAL,
-							  NULL
-			);
-		if (outfile == INVALID_HANDLE_VALUE)
-		{
-			rprintf ("outfile creation failed\n");
-			return;
-		}
-
-		saveStdout = GetStdHandle (STD_OUTPUT_HANDLE);
-
-		if (!SetStdHandle (STD_OUTPUT_HANDLE, outfile))
-		{
-			rprintf ("could not redirect output\n");
-			return;
-		}
-		rprintf ("redirection of output ok\n");
-
-	}
-	else
-	{
-		rprintf ("out == NULL\n");
-		outfile = NULL;
-	}
-
-	if (err != NULL)
-	{
-		errfile = CreateFile (err,
-							  GENERIC_WRITE,
-							  0,
-							  &sa,
-							  CREATE_ALWAYS,
-							  FILE_ATTRIBUTE_NORMAL,
-							  NULL
-			);
-		if (errfile == INVALID_HANDLE_VALUE)
-		{
-			rprintf ("errfile creation failed\n");
-			return;
-		}
-
-		saveStderr = GetStdHandle (STD_ERROR_HANDLE);
-
-		if (!SetStdHandle (STD_ERROR_HANDLE, errfile))
-		{
-			rprintf ("could not redirect errput\n");
-			return;
-		}
-		rprintf ("redirection of errors ok\n");
-	}
-	else
-	{
-		rprintf ("err == NULL\n");
-		errfile = NULL;
-	}
-
-	si.cb = sizeof (STARTUPINFO);
-	si.lpReserved = NULL;
-	si.lpReserved2 = NULL;
-	si.cbReserved2 = 0;
-	si.lpDesktop = NULL;
-	si.lpTitle = NULL;
-	si.dwFlags = 0;
-
-	fsuccess =
-		CreateProcess (NULL,				/* pointer to name of executable module		*/
-					   commandline,			/* pointer to command line string			*/
-					   NULL,				/* pointer to process security attributes	*/
-					   NULL,				/* pointer to thread security attributes	*/
-					   TRUE,				/* handle inheritance flag					*/
-					   DETACHED_PROCESS,	/* creation flags							*/
-					   env,					/* pointer to new environment block			*/
-					   dir,					/* pointer to current directory name		*/
-					   &si,					/* pointer to STARTUPINFO					*/
-					   &pi					/* pointer to PROCESS_INFORMATION			*/
-		);
-	if (fsuccess)
-	{
-		rprintf ("WCP: success\n");
-		WaitForSingleObject (pi.hProcess, INFINITE);
-		GetExitCodeProcess (pi.hProcess, (unsigned long *) exitcode);
-		rprintf ("WCP: exitcode = %d\n", *exitcode);
-		*success = TRUE;
-	}
-	else
-	{
-		rprintf ("WCP: failure %d\n", (int) GetLastError ());
-		*success = FALSE;
-		*exitcode = -1;
-	}
-
-	if (infile != NULL)
-	{
-		if (SetStdHandle (STD_INPUT_HANDLE, saveStdin))
-			rprintf ("resetting stdin ok\n");
-		else
-			rprintf ("resetting stdin failed\n");
-
-		if (CloseHandle (infile))
-			rprintf ("closing infile ok\n");
-		else
-			rprintf ("closing infile failed\n");
-	}
-	else
-		rprintf ("no need to close and reset input\n");
-
-	if (outfile != NULL)
-	{
-		if (SetStdHandle (STD_OUTPUT_HANDLE, saveStdout))
-			rprintf ("resetting stdout ok\n");
-		else
-			rprintf ("resetting stdout failed\n");
-
-		if (CloseHandle (outfile))
-			rprintf ("closing outfile ok\n");
-		else
-			rprintf ("closing outfile failed\n");
-	}
-	else
-		rprintf ("no need to close and reset output\n");
-
-	if (errfile != NULL)
-	{
-		if (SetStdHandle (STD_ERROR_HANDLE, saveStderr))
-			rprintf ("resetting stderr ok\n");
-		else
-			rprintf ("resetting stderr failed\n");
-
-		if (CloseHandle (errfile))
-			rprintf ("closing errfile ok\n");
-		else
-			rprintf ("closing errfile failed\n");
-	}
-	else
-		rprintf ("no need to close and reset errput\n");
-
-	rprintf ("WCP: returning\n");
-	/* *oos = ios; */
-}	/* WinCallProcess */
-
-void WinLaunchApp (CLEAN_STRING commandline, BOOL console, OS ios, Bool * success, OS * oos)
-{
-	STARTUPINFO si;
-	PROCESS_INFORMATION pi;
-	BOOL fsuccess;
-	char path[MAX_PATH];
-	char *cl, *thepath;
-	int i;
-	DWORD error;
-
-	rprintf ("WLA: starting...\n");
-
-	*success = FALSE;
-	*oos = ios;
-	si.lpTitle = NULL;
-
-	rprintf ("WLA: step 2.\n");
-
-	cl = cstring (commandline);
-	lstrcpy (path, cl);
-
-	for (i = lstrlen (path); path[i] != '\\' && i >= 0; i--)
-		path[i] = 0;
-
-	if (i == 0)
-		thepath = NULL;
-	else
-	{	/* path[i] = '\"'; */
-		thepath = path + 1;
-	}
-
-	rprintf ("WLA: step 2a: directory = <%s>\n", thepath);
-
-	rprintf ("WLA: step 3: filling in si.\n");
-
-	si.cb = sizeof (STARTUPINFO);
-	si.lpReserved = NULL;
-	si.lpReserved2 = NULL;
-	si.cbReserved2 = 0;
-	si.lpDesktop = NULL;
-	si.dwFlags = 0;
-	si.lpTitle = NULL;
-
-	rprintf ("WLA: step 4: calling process \"%s\".\n", cl);
-
-	fsuccess =
-		CreateProcess (NULL,	/* pointer to name of executable module		*/
-					   cl,		/* pointer to command line string			*/
-					   NULL,	/* pointer to process security attributes	*/
-					   NULL,	/* pointer to thread security attributes	*/
-					   TRUE,	/* handle inheritance flag					*/
-					   0,		/* creation flags							*/
-					   NULL,	/* pointer to new environment block			*/
-					   thepath, /* pointer to current directory name		*/
-					   &si, 	/* pointer to STARTUPINFO					*/
-					   &pi		/* pointer to PROCESS_INFORMATION			*/
-		);
-	error = GetLastError ();
-	if (fsuccess)
-	{
-		rprintf ("WLA: success\n");
-	}
-	else
-	{
-		rprintf ("WLA: failure %d\n", error);
-	}
-
-	rprintf ("WLA: step 5: returning\n");
-	*success = fsuccess;
-	*oos = ios;
-	rprintf ("WLA: done...\n");
-}	/* WinLaunchApp */
-
-/*	New version of WinLaunchApp. In addition to the commandline, it gets a directory argument.
-*/
-void WinLaunchApp2 (CLEAN_STRING commandline, CLEAN_STRING pathname, BOOL console, OS ios, Bool * success, OS * oos)
-{
-	STARTUPINFO si;
-	PROCESS_INFORMATION pi;
-	BOOL fsuccess;
-	char commandline2[MAX_PATH];
-	char *cl, *thepath;
-	DWORD error;
-
-	rprintf ("WLA2: starting...\n");
-
-	*success = FALSE;
-	*oos = ios;
-	si.lpTitle = NULL;
-
-	rprintf ("WLA2: step 2.\n");
-
-	cl = cstring (commandline);
-	lstrcpy (commandline2, cl);
-	thepath = cstring (pathname);
-
-	rprintf ("WLA2: step 3: filling in si.\n");
-
-	si.cb = sizeof (STARTUPINFO);
-	si.lpReserved = NULL;
-	si.lpReserved2 = NULL;
-	si.cbReserved2 = 0;
-	si.lpDesktop = NULL;
-	si.dwFlags = 0;
-	si.lpTitle = NULL;
-
-	rprintf ("WLA2: step 4: calling process \"%s\".\n", cl);
-
-	fsuccess =
-		CreateProcess (NULL,		/* pointer to name of executable module		*/
-					   commandline2,/* pointer to command line string			*/
-					   NULL,		/* pointer to process security attributes	*/
-					   NULL,		/* pointer to thread security attributes	*/
-					   TRUE,		/* handle inheritance flag					*/
-					   0,			/* creation flags							*/
-					   NULL,		/* pointer to new environment block			*/
-					   thepath,		/* pointer to current directory name		*/
-					   &si,			/* pointer to STARTUPINFO					*/
-					   &pi			/* pointer to PROCESS_INFORMATION			*/
-		);
-	error = GetLastError ();
-	if (fsuccess)
-	{
-		rprintf ("WLA2: success\n");
-	}
-	else
-	{
-		rprintf ("WLA2: failure %d\n", error);
-	}
-
-	rprintf ("WLA2: step 5: returning\n");
-	*success = fsuccess;
-	*oos = ios;
-	rprintf ("WLA2: done...\n");
-}	/* WinLaunchApp2 */
-
-
-CLEAN_STRING WinGetModulePath (void)
-{
-	char path[MAX_PATH + 1];
-
-	GetModuleFileName (NULL, path, MAX_PATH);
-
-	return cleanstring (path);
-}	/* WinGetModulePath */
-
-void WinFileModifiedDate (CLEAN_STRING name, Bool *exists, int *yy, int *mm, int *dd, int *h, int *m, int *s)
-{
-	char *file_name;
-	HANDLE handle;
-	WIN32_FIND_DATA find_data;
-
-	file_name = cstring (name);
-
-	handle = FindFirstFile (file_name, &find_data);
-
-	if (handle != INVALID_HANDLE_VALUE)
-	{
-		SYSTEMTIME system_time;
-
-		FindClose (handle);
-
-		if (FileTimeToSystemTime (&find_data.ftLastWriteTime, &system_time))
-		{
-			*exists = TRUE;
-			*yy = system_time.wYear;
-			*mm = system_time.wMonth;
-			*dd = system_time.wDay;
-			*h = system_time.wHour;
-			*m = system_time.wMinute;
-			*s = system_time.wSecond;
-			return;
-		}
-	}
-
-	*exists = FALSE;
-	*yy = 0;
-	*mm = 0;
-	*dd = 0;
-	*h = 0;
-	*m = 0;
-	*s = 0;
-}	/* WinFileModifiedDate */
