@@ -11,10 +11,14 @@
  * to the current environment so that the normal $getenv$ and $exec*$ functions
  * should continue to work properly.
  */
-
 #include "Rts.h"
 #include "HsPosix.h"
 
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
+#include <stdio.h>
+
 /* Switch this on once we've moved the environment to the malloc arena */
 int dirtyEnv = 0;
 
