@@ -1,8 +1,3 @@
-{-# OPTIONS -#include <windows.h> #-}
-{-# OPTIONS -#include "errors.h" #-}
-{-# OPTIONS -#include "win32debug.h" #-}
-{-# OPTIONS -#include "diatemp.h" #-}
-{-# OPTIONS -#include "Win32Dialogue_stub_ffi.h" #-}
 {-# OPTIONS -#include "Win32Dialogue_stub.h" #-}
 module Win32Dialogue where
 
@@ -12,6 +7,8 @@ import GDITypes
 import Win32Window
 import Win32WinMessage
 import Win32Control
+import Foreign.ForeignPtr
+import Foreign.Ptr ( castFunPtrToPtr )
 
 
 type DTemplate = LPCTSTR
@@ -30,15 +27,21 @@ mkResource arg1 =
   prim_mkResource arg1
   >>= \  res1  ->
   (return (res1))
-foreign import  ccall "prim_mkResource" unsafe prim_mkResource :: Int -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_mkResource" prim_mkResource :: Int -> IO (Addr)
 
 mkDialogTemplateFromResource :: Int -> IO DTemplate
 mkDialogTemplateFromResource = mkResource
 
 type DialogProc = HWND -> WindowMessage -> WPARAM -> LPARAM -> IO Int
+type DialogProc_FFI = Ptr () -> WindowMessage -> WPARAM -> LPARAM -> IO Int
 
 
-foreign export stdcall dynamic marshall_dialogProc_ :: DialogProc -> IO Addr
+marshall_dialogProc_ :: DialogProc -> IO (FunPtr DialogProc)
+marshall_dialogProc_ cl = mkDialogClosure (\ x -> cl (ptrToAddr x))
+
+-- ToDo: this was declared as a stdcall not a ccall - let's
+-- hope and pray that it makes no difference - ADR
+foreign import ccall "wrapper" mkDialogClosure :: DialogProc_FFI -> IO (FunPtr DialogProc)
 --marshall_dialogProc_ x = return nullAddr
 
 dialogBox :: HINSTANCE -> DTemplate -> MbHWND -> DialogProc -> IO Int
@@ -56,19 +59,18 @@ dialogBox hInst lpTemp hWndParent gc_arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (res1))
-foreign import  ccall "prim_dialogBox" prim_dialogBox :: Addr -> Addr -> Addr -> Addr -> IO (Addr)
-foreign import ccall "access_prim_dialogBox_res1" unsafe access_prim_dialogBox_res1 :: Addr -> IO (Int)
-foreign import ccall "access_prim_dialogBox_gc_failed" unsafe access_prim_dialogBox_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_dialogBox_gc_failstring" unsafe access_prim_dialogBox_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_dialogBox" prim_dialogBox :: Addr -> Addr -> Addr -> FunPtr DialogProc -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBox_res1 :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBox_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBox_gc_failstring :: Addr -> IO (Addr)
 
 dialogBoxParam :: HINSTANCE -> DTemplate -> MbHWND -> DialogProc -> LPARAM -> IO Int
-dialogBoxParam hInst lpTemp hWndParent gc_arg1 gc_arg2 =
+dialogBoxParam hInst lpTemp hWndParent gc_arg1 dwInit =
   (case hWndParent of {
       Nothing -> (return (nullHANDLE));
       (Just hWndParent) -> (return ((hWndParent)))
    }) >>= \ (hWndParent) ->
   (marshall_dialogProc_ gc_arg1) >>= \ (diaFun) ->
-  case ( int32ToInt  gc_arg2) of { dwInit ->
   prim_dialogBoxParam hInst lpTemp hWndParent diaFun dwInit
   >>= \ gc_result ->
   access_prim_dialogBoxParam_res1 (gc_result :: Addr) >>= \ res1 ->
@@ -76,11 +78,11 @@ dialogBoxParam hInst lpTemp hWndParent gc_arg1 gc_arg2 =
   access_prim_dialogBoxParam_gc_failstring (gc_result :: Addr) >>= \ gc_failstring ->
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
-  else (return (res1))}
-foreign import  ccall "prim_dialogBoxParam" prim_dialogBoxParam :: Addr -> Addr -> Addr -> Addr -> Int -> IO (Addr)
-foreign import ccall "access_prim_dialogBoxParam_res1" unsafe access_prim_dialogBoxParam_res1 :: Addr -> IO (Int)
-foreign import ccall "access_prim_dialogBoxParam_gc_failed" unsafe access_prim_dialogBoxParam_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_dialogBoxParam_gc_failstring" unsafe access_prim_dialogBoxParam_gc_failstring :: Addr -> IO (Addr)
+  else (return (res1))
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_dialogBoxParam" prim_dialogBoxParam :: Addr -> Addr -> Addr -> FunPtr DialogProc -> Int32 -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBoxParam_res1 :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBoxParam_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBoxParam_gc_failstring :: Addr -> IO (Addr)
 
 dialogBoxIndirect :: HINSTANCE -> DTemplateMem -> MbHWND -> DialogProc -> IO Int
 dialogBoxIndirect hInst lpTemp hWndParent gc_arg1 =
@@ -97,19 +99,18 @@ dialogBoxIndirect hInst lpTemp hWndParent gc_arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (res1))
-foreign import  ccall "prim_dialogBoxIndirect" prim_dialogBoxIndirect :: Addr -> Addr -> Addr -> Addr -> IO (Addr)
-foreign import ccall "access_prim_dialogBoxIndirect_res1" unsafe access_prim_dialogBoxIndirect_res1 :: Addr -> IO (Int)
-foreign import ccall "access_prim_dialogBoxIndirect_gc_failed" unsafe access_prim_dialogBoxIndirect_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_dialogBoxIndirect_gc_failstring" unsafe access_prim_dialogBoxIndirect_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_dialogBoxIndirect" prim_dialogBoxIndirect :: Addr -> Addr -> Addr -> FunPtr DialogProc -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBoxIndirect_res1 :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBoxIndirect_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBoxIndirect_gc_failstring :: Addr -> IO (Addr)
 
 dialogBoxIndirectParam :: HINSTANCE -> DTemplateMem -> MbHWND -> DialogProc -> LPARAM -> IO Int
-dialogBoxIndirectParam hInst lpTemp hWndParent gc_arg1 gc_arg2 =
+dialogBoxIndirectParam hInst lpTemp hWndParent gc_arg1 dwInit =
   (case hWndParent of {
       Nothing -> (return (nullHANDLE));
       (Just hWndParent) -> (return ((hWndParent)))
    }) >>= \ (hWndParent) ->
   (marshall_dialogProc_ gc_arg1) >>= \ (diaFun) ->
-  case ( int32ToInt  gc_arg2) of { dwInit ->
   prim_dialogBoxIndirectParam hInst lpTemp hWndParent diaFun dwInit
   >>= \ gc_result ->
   access_prim_dialogBoxIndirectParam_res1 (gc_result :: Addr) >>= \ res1 ->
@@ -117,11 +118,11 @@ dialogBoxIndirectParam hInst lpTemp hWndParent gc_arg1 gc_arg2 =
   access_prim_dialogBoxIndirectParam_gc_failstring (gc_result :: Addr) >>= \ gc_failstring ->
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
-  else (return (res1))}
-foreign import  ccall "prim_dialogBoxIndirectParam" prim_dialogBoxIndirectParam :: Addr -> Addr -> Addr -> Addr -> Int -> IO (Addr)
-foreign import ccall "access_prim_dialogBoxIndirectParam_res1" unsafe access_prim_dialogBoxIndirectParam_res1 :: Addr -> IO (Int)
-foreign import ccall "access_prim_dialogBoxIndirectParam_gc_failed" unsafe access_prim_dialogBoxIndirectParam_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_dialogBoxIndirectParam_gc_failstring" unsafe access_prim_dialogBoxIndirectParam_gc_failstring :: Addr -> IO (Addr)
+  else (return (res1))
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_dialogBoxIndirectParam" prim_dialogBoxIndirectParam :: Addr -> Addr -> Addr -> FunPtr DialogProc -> Int32 -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBoxIndirectParam_res1 :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBoxIndirectParam_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_dialogBoxIndirectParam_gc_failstring :: Addr -> IO (Addr)
 
 
 data DialogTemplate
@@ -215,14 +216,14 @@ getFinalDialog arg1 =
   prim_getFinalDialog arg1
   >>= \  res1  ->
   (return (res1))
-foreign import  ccall "prim_getFinalDialog" unsafe prim_getFinalDialog :: Addr -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_getFinalDialog" prim_getFinalDialog :: Addr -> IO (Addr)
 
 mkDiaTemplate :: Int -> Int -> Int -> Int -> Int -> WindowStyle -> DWORD -> Addr -> Addr -> Addr -> Addr -> Int -> IO Addr
 mkDiaTemplate arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9 arg10 arg11 arg12 =
   prim_mkDiaTemplate arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9 arg10 arg11 arg12
   >>= \  res1  ->
   (return (res1))
-foreign import  ccall "prim_mkDiaTemplate" unsafe prim_mkDiaTemplate :: Int -> Int -> Int -> Int -> Int -> Word32 -> Word32 -> Addr -> Addr -> Addr -> Addr -> Int -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_mkDiaTemplate" prim_mkDiaTemplate :: Int -> Int -> Int -> Int -> Int -> Word32 -> Word32 -> Addr -> Addr -> Addr -> Addr -> Int -> IO (Addr)
 
 addControl :: Addr -> DialogControl -> IO ()
 addControl dtemp (DialogControl x y cx cy mb_text mb_class
@@ -239,7 +240,7 @@ addDiaControl arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9 arg10 =
   prim_addDiaControl arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9 arg10
   >>= \  res1  ->
   (return (res1))
-foreign import  ccall "prim_addDiaControl" unsafe prim_addDiaControl :: Addr -> Addr -> Int -> Addr -> Word32 -> Int -> Int -> Int -> Int -> Word32 -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_addDiaControl" prim_addDiaControl :: Addr -> Addr -> Int -> Addr -> Word32 -> Int -> Int -> Int -> Int -> Word32 -> IO (Addr)
 
 marshall_res :: Either ResourceID String -> IO Addr
 marshall_res (Left r)  = mkResource r
@@ -251,7 +252,7 @@ toUnicodeStr gc_arg1 =
   prim_toUnicodeStr arg1
   >>= \  gc_res1  ->
   (return (gc_res1))
-foreign import  ccall "prim_toUnicodeStr" unsafe prim_toUnicodeStr :: Addr -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_toUnicodeStr" prim_toUnicodeStr :: Addr -> IO (Addr)
 
 -- modeless dialogs
 createDialog :: HINSTANCE -> DTemplate -> MbHWND -> DialogProc -> IO HWND
@@ -269,19 +270,18 @@ createDialog hInst lpTemp hWndParent gc_arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (res1))
-foreign import  ccall "prim_createDialog" prim_createDialog :: Addr -> Addr -> Addr -> Addr -> IO (Addr)
-foreign import ccall "access_prim_createDialog_res1" unsafe access_prim_createDialog_res1 :: Addr -> IO (Addr)
-foreign import ccall "access_prim_createDialog_gc_failed" unsafe access_prim_createDialog_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_createDialog_gc_failstring" unsafe access_prim_createDialog_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_createDialog" prim_createDialog :: Addr -> Addr -> Addr -> FunPtr DialogProc -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialog_res1 :: Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialog_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialog_gc_failstring :: Addr -> IO (Addr)
 
 createDialogParam :: HINSTANCE -> DTemplate -> MbHWND -> DialogProc -> LPARAM -> IO HWND
-createDialogParam hInst lpTemp hWndParent gc_arg1 gc_arg2 =
+createDialogParam hInst lpTemp hWndParent gc_arg1 dwInit =
   (case hWndParent of {
       Nothing -> (return (nullHANDLE));
       (Just hWndParent) -> (return ((hWndParent)))
    }) >>= \ (hWndParent) ->
   (marshall_dialogProc_ gc_arg1) >>= \ (diaFun) ->
-  case ( int32ToInt  gc_arg2) of { dwInit ->
   prim_createDialogParam hInst lpTemp hWndParent diaFun dwInit
   >>= \ gc_result ->
   access_prim_createDialogParam_res1 (gc_result :: Addr) >>= \ res1 ->
@@ -289,11 +289,11 @@ createDialogParam hInst lpTemp hWndParent gc_arg1 gc_arg2 =
   access_prim_createDialogParam_gc_failstring (gc_result :: Addr) >>= \ gc_failstring ->
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
-  else (return (res1))}
-foreign import  ccall "prim_createDialogParam" prim_createDialogParam :: Addr -> Addr -> Addr -> Addr -> Int -> IO (Addr)
-foreign import ccall "access_prim_createDialogParam_res1" unsafe access_prim_createDialogParam_res1 :: Addr -> IO (Addr)
-foreign import ccall "access_prim_createDialogParam_gc_failed" unsafe access_prim_createDialogParam_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_createDialogParam_gc_failstring" unsafe access_prim_createDialogParam_gc_failstring :: Addr -> IO (Addr)
+  else (return (res1))
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_createDialogParam" prim_createDialogParam :: Addr -> Addr -> Addr -> FunPtr DialogProc -> Int32 -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialogParam_res1 :: Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialogParam_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialogParam_gc_failstring :: Addr -> IO (Addr)
 
 createDialogIndirect :: HINSTANCE -> DTemplateMem -> MbHWND -> DialogProc -> IO HWND
 createDialogIndirect hInst lpTemp hWndParent gc_arg1 =
@@ -310,19 +310,18 @@ createDialogIndirect hInst lpTemp hWndParent gc_arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (res1))
-foreign import  ccall "prim_createDialogIndirect" prim_createDialogIndirect :: Addr -> Addr -> Addr -> Addr -> IO (Addr)
-foreign import ccall "access_prim_createDialogIndirect_res1" unsafe access_prim_createDialogIndirect_res1 :: Addr -> IO (Addr)
-foreign import ccall "access_prim_createDialogIndirect_gc_failed" unsafe access_prim_createDialogIndirect_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_createDialogIndirect_gc_failstring" unsafe access_prim_createDialogIndirect_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_createDialogIndirect" prim_createDialogIndirect :: Addr -> Addr -> Addr -> FunPtr DialogProc -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialogIndirect_res1 :: Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialogIndirect_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialogIndirect_gc_failstring :: Addr -> IO (Addr)
 
 createDialogIndirectParam :: HINSTANCE -> DTemplateMem -> MbHWND -> DialogProc -> LPARAM -> IO HWND
-createDialogIndirectParam hInst lpTemp hWndParent gc_arg1 gc_arg2 =
+createDialogIndirectParam hInst lpTemp hWndParent gc_arg1 dwInit =
   (case hWndParent of {
       Nothing -> (return (nullHANDLE));
       (Just hWndParent) -> (return ((hWndParent)))
    }) >>= \ (hWndParent) ->
   (marshall_dialogProc_ gc_arg1) >>= \ (diaFun) ->
-  case ( int32ToInt  gc_arg2) of { dwInit ->
   prim_createDialogIndirectParam hInst lpTemp hWndParent diaFun dwInit
   >>= \ gc_result ->
   access_prim_createDialogIndirectParam_res1 (gc_result :: Addr) >>= \ res1 ->
@@ -330,24 +329,22 @@ createDialogIndirectParam hInst lpTemp hWndParent gc_arg1 gc_arg2 =
   access_prim_createDialogIndirectParam_gc_failstring (gc_result :: Addr) >>= \ gc_failstring ->
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
-  else (return (res1))}
-foreign import  ccall "prim_createDialogIndirectParam" prim_createDialogIndirectParam :: Addr -> Addr -> Addr -> Addr -> Int -> IO (Addr)
-foreign import ccall "access_prim_createDialogIndirectParam_res1" unsafe access_prim_createDialogIndirectParam_res1 :: Addr -> IO (Addr)
-foreign import ccall "access_prim_createDialogIndirectParam_gc_failed" unsafe access_prim_createDialogIndirectParam_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_createDialogIndirectParam_gc_failstring" unsafe access_prim_createDialogIndirectParam_gc_failstring :: Addr -> IO (Addr)
+  else (return (res1))
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_createDialogIndirectParam" prim_createDialogIndirectParam :: Addr -> Addr -> Addr -> FunPtr DialogProc -> Int32 -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialogIndirectParam_res1 :: Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialogIndirectParam_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_createDialogIndirectParam_gc_failstring :: Addr -> IO (Addr)
 
 defDlgProc :: MbHWND -> WindowMessage -> WPARAM -> LPARAM -> IO LRESULT
-defDlgProc arg1 arg2 arg3 gc_arg1 =
+defDlgProc arg1 arg2 arg3 arg4 =
   (case arg1 of {
       Nothing -> (return (nullHANDLE));
       (Just arg1) -> (return ((arg1)))
    }) >>= \ (arg1) ->
-  case ( int32ToInt  gc_arg1) of { arg4 ->
   prim_defDlgProc arg1 arg2 arg3 arg4
   >>= \  res1  ->
-  let gc_res1 = ( intToInt32  (res1)) in
-  (return (gc_res1))}
-foreign import  ccall "prim_defDlgProc" prim_defDlgProc :: Addr -> Word32 -> Word32 -> Int -> IO (Int)
+  (return (res1))
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_defDlgProc" prim_defDlgProc :: Addr -> Word32 -> Word32 -> Int32 -> IO (Int32)
 
 endDialog :: HWND -> Int -> IO BOOL
 endDialog arg1 arg2 =
@@ -360,18 +357,17 @@ endDialog arg1 arg2 =
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (unmarshall_bool_ res1) >>= \ gc_res1 ->
        (return (gc_res1))
-foreign import  ccall "prim_endDialog" prim_endDialog :: Addr -> Int -> IO (Addr)
-foreign import ccall "access_prim_endDialog_res1" unsafe access_prim_endDialog_res1 :: Addr -> IO (Int)
-foreign import ccall "access_prim_endDialog_gc_failed" unsafe access_prim_endDialog_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_endDialog_gc_failstring" unsafe access_prim_endDialog_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_endDialog" prim_endDialog :: Addr -> Int -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_endDialog_res1 :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_endDialog_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_endDialog_gc_failstring :: Addr -> IO (Addr)
 
 getDialogBaseUnits :: IO LONG
 getDialogBaseUnits =
   prim_getDialogBaseUnits
   >>= \  res1  ->
-  let gc_res1 = ( intToInt32  (res1)) in
-  (return (gc_res1))
-foreign import  ccall "prim_getDialogBaseUnits" unsafe prim_getDialogBaseUnits :: IO (Int)
+  (return (res1))
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_getDialogBaseUnits" prim_getDialogBaseUnits :: IO (Int32)
 
 getDlgCtrlID :: HWND -> IO Int
 getDlgCtrlID arg1 =
@@ -383,10 +379,10 @@ getDlgCtrlID arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (res1))
-foreign import  ccall "prim_getDlgCtrlID" unsafe prim_getDlgCtrlID :: Addr -> IO (Addr)
-foreign import ccall "access_prim_getDlgCtrlID_res1" unsafe access_prim_getDlgCtrlID_res1 :: Addr -> IO (Int)
-foreign import ccall "access_prim_getDlgCtrlID_gc_failed" unsafe access_prim_getDlgCtrlID_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_getDlgCtrlID_gc_failstring" unsafe access_prim_getDlgCtrlID_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_getDlgCtrlID" prim_getDlgCtrlID :: Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgCtrlID_res1 :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgCtrlID_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgCtrlID_gc_failstring :: Addr -> IO (Addr)
 
 getDlgItem :: HWND -> Int -> IO HWND
 getDlgItem arg1 arg2 =
@@ -398,10 +394,10 @@ getDlgItem arg1 arg2 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (res1))
-foreign import  ccall "prim_getDlgItem" unsafe prim_getDlgItem :: Addr -> Int -> IO (Addr)
-foreign import ccall "access_prim_getDlgItem_res1" unsafe access_prim_getDlgItem_res1 :: Addr -> IO (Addr)
-foreign import ccall "access_prim_getDlgItem_gc_failed" unsafe access_prim_getDlgItem_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_getDlgItem_gc_failstring" unsafe access_prim_getDlgItem_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_getDlgItem" prim_getDlgItem :: Addr -> Int -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgItem_res1 :: Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgItem_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgItem_gc_failstring :: Addr -> IO (Addr)
 
 getDlgItemInt :: HWND -> Int -> Bool -> IO Int
 getDlgItemInt arg1 arg2 gc_arg1 =
@@ -414,10 +410,10 @@ getDlgItemInt arg1 arg2 gc_arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (res1))
-foreign import  ccall "prim_getDlgItemInt" prim_getDlgItemInt :: Addr -> Int -> Int -> IO (Addr)
-foreign import ccall "access_prim_getDlgItemInt_res1" unsafe access_prim_getDlgItemInt_res1 :: Addr -> IO (Int)
-foreign import ccall "access_prim_getDlgItemInt_gc_failed" unsafe access_prim_getDlgItemInt_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_getDlgItemInt_gc_failstring" unsafe access_prim_getDlgItemInt_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_getDlgItemInt" prim_getDlgItemInt :: Addr -> Int -> Int -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgItemInt_res1 :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgItemInt_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgItemInt_gc_failstring :: Addr -> IO (Addr)
 
 getDlgItemText :: HWND -> Int -> Int -> IO String
 getDlgItemText arg1 arg2 arg3 =
@@ -430,10 +426,10 @@ getDlgItemText arg1 arg2 arg3 =
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (unmarshall_string_ gc_res2) >>= \ gc_res1 ->
        (return (gc_res1))
-foreign import  ccall "prim_getDlgItemText" prim_getDlgItemText :: Addr -> Int -> Int -> IO (Addr)
-foreign import ccall "access_prim_getDlgItemText_gc_res2" unsafe access_prim_getDlgItemText_gc_res2 :: Addr -> IO (Addr)
-foreign import ccall "access_prim_getDlgItemText_gc_failed" unsafe access_prim_getDlgItemText_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_getDlgItemText_gc_failstring" unsafe access_prim_getDlgItemText_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_getDlgItemText" prim_getDlgItemText :: Addr -> Int -> Int -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgItemText_gc_res2 :: Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgItemText_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getDlgItemText_gc_failstring :: Addr -> IO (Addr)
 
 getNextDlgGroupItem :: HWND -> HWND -> BOOL -> IO HWND
 getNextDlgGroupItem arg1 arg2 gc_arg1 =
@@ -446,10 +442,10 @@ getNextDlgGroupItem arg1 arg2 gc_arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (res1))
-foreign import  ccall "prim_getNextDlgGroupItem" unsafe prim_getNextDlgGroupItem :: Addr -> Addr -> Int -> IO (Addr)
-foreign import ccall "access_prim_getNextDlgGroupItem_res1" unsafe access_prim_getNextDlgGroupItem_res1 :: Addr -> IO (Addr)
-foreign import ccall "access_prim_getNextDlgGroupItem_gc_failed" unsafe access_prim_getNextDlgGroupItem_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_getNextDlgGroupItem_gc_failstring" unsafe access_prim_getNextDlgGroupItem_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_getNextDlgGroupItem" prim_getNextDlgGroupItem :: Addr -> Addr -> Int -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getNextDlgGroupItem_res1 :: Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getNextDlgGroupItem_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getNextDlgGroupItem_gc_failstring :: Addr -> IO (Addr)
 
 getNextDlgTabItem :: HWND -> HWND -> BOOL -> IO HWND
 getNextDlgTabItem arg1 arg2 gc_arg1 =
@@ -462,10 +458,10 @@ getNextDlgTabItem arg1 arg2 gc_arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (res1))
-foreign import  ccall "prim_getNextDlgTabItem" unsafe prim_getNextDlgTabItem :: Addr -> Addr -> Int -> IO (Addr)
-foreign import ccall "access_prim_getNextDlgTabItem_res1" unsafe access_prim_getNextDlgTabItem_res1 :: Addr -> IO (Addr)
-foreign import ccall "access_prim_getNextDlgTabItem_gc_failed" unsafe access_prim_getNextDlgTabItem_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_getNextDlgTabItem_gc_failstring" unsafe access_prim_getNextDlgTabItem_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_getNextDlgTabItem" prim_getNextDlgTabItem :: Addr -> Addr -> Int -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getNextDlgTabItem_res1 :: Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getNextDlgTabItem_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_getNextDlgTabItem_gc_failstring :: Addr -> IO (Addr)
 
 isDialogMessage :: HWND -> LPMSG -> IO BOOL
 isDialogMessage arg1 arg2 =
@@ -473,7 +469,7 @@ isDialogMessage arg1 arg2 =
   >>= \  res1  ->
   (unmarshall_bool_ res1) >>= \ gc_res1 ->
   (return (gc_res1))
-foreign import  ccall "prim_isDialogMessage" prim_isDialogMessage :: Addr -> Addr -> IO (Int)
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_isDialogMessage" prim_isDialogMessage :: Addr -> Addr -> IO (Int)
 
 mapDialogRect :: HWND -> LPRECT -> IO ()
 mapDialogRect arg1 arg2 =
@@ -484,20 +480,18 @@ mapDialogRect arg1 arg2 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (()))
-foreign import  ccall "prim_mapDialogRect" unsafe prim_mapDialogRect :: Addr -> Addr -> IO (Addr)
-foreign import ccall "access_prim_mapDialogRect_gc_failed" unsafe access_prim_mapDialogRect_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_mapDialogRect_gc_failstring" unsafe access_prim_mapDialogRect_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_mapDialogRect" prim_mapDialogRect :: Addr -> Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_mapDialogRect_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_mapDialogRect_gc_failstring :: Addr -> IO (Addr)
 
 -- No MessageBox* funs in here just yet.
 
 sendDlgItemMessage :: HWND -> Int -> WindowMessage -> WPARAM -> LPARAM -> IO LONG
-sendDlgItemMessage arg1 arg2 arg3 arg4 gc_arg1 =
-  case ( int32ToInt  gc_arg1) of { arg5 ->
+sendDlgItemMessage arg1 arg2 arg3 arg4 arg5 =
   prim_sendDlgItemMessage arg1 arg2 arg3 arg4 arg5
   >>= \  res1  ->
-  let gc_res1 = ( intToInt32  (res1)) in
-  (return (gc_res1))}
-foreign import  ccall "prim_sendDlgItemMessage" prim_sendDlgItemMessage :: Addr -> Int -> Word32 -> Word32 -> Int -> IO (Int)
+  (return (res1))
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_sendDlgItemMessage" prim_sendDlgItemMessage :: Addr -> Int -> Word32 -> Word32 -> Int32 -> IO (Int32)
 
 setDlgItemInt :: HWND -> Int -> UINT -> BOOL -> IO ()
 setDlgItemInt arg1 arg2 arg3 gc_arg1 =
@@ -509,9 +503,9 @@ setDlgItemInt arg1 arg2 arg3 gc_arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (()))
-foreign import  ccall "prim_setDlgItemInt" unsafe prim_setDlgItemInt :: Addr -> Int -> Word32 -> Int -> IO (Addr)
-foreign import ccall "access_prim_setDlgItemInt_gc_failed" unsafe access_prim_setDlgItemInt_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_setDlgItemInt_gc_failstring" unsafe access_prim_setDlgItemInt_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_setDlgItemInt" prim_setDlgItemInt :: Addr -> Int -> Word32 -> Int -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_setDlgItemInt_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_setDlgItemInt_gc_failstring :: Addr -> IO (Addr)
 
 setDlgItemText :: HWND -> Int -> String -> IO ()
 setDlgItemText arg1 arg2 gc_arg1 =
@@ -523,144 +517,30 @@ setDlgItemText arg1 arg2 gc_arg1 =
   if ( gc_failed /= (0::Int))
   then unmarshall_string_ gc_failstring >>=  ioError  . userError
   else (return (()))
-foreign import  ccall "prim_setDlgItemText" prim_setDlgItemText :: Addr -> Int -> Addr -> IO (Addr)
-foreign import ccall "access_prim_setDlgItemText_gc_failed" unsafe access_prim_setDlgItemText_gc_failed :: Addr -> IO (Int)
-foreign import ccall "access_prim_setDlgItemText_gc_failstring" unsafe access_prim_setDlgItemText_gc_failstring :: Addr -> IO (Addr)
+foreign import  ccall "Win32Dialogue_stub_ffi.h prim_setDlgItemText" prim_setDlgItemText :: Addr -> Int -> Addr -> IO (Addr)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_setDlgItemText_gc_failed :: Addr -> IO (Int)
+foreign import ccall unsafe "Win32Dialogue_stub_ffi.h" access_prim_setDlgItemText_gc_failstring :: Addr -> IO (Addr)
 
-dS_3DLOOK :: WindowStyle
-dS_3DLOOK =
-  unsafePerformIO(
-    prim_dS_3DLOOK
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_3DLOOK" unsafe prim_dS_3DLOOK :: IO (Word32)
-dS_ABSALIGN :: WindowStyle
-dS_ABSALIGN =
-  unsafePerformIO(
-    prim_dS_ABSALIGN
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_ABSALIGN" unsafe prim_dS_ABSALIGN :: IO (Word32)
-dS_CENTER :: WindowStyle
-dS_CENTER =
-  unsafePerformIO(
-    prim_dS_CENTER
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_CENTER" unsafe prim_dS_CENTER :: IO (Word32)
-dS_CENTERMOUSE :: WindowStyle
-dS_CENTERMOUSE =
-  unsafePerformIO(
-    prim_dS_CENTERMOUSE
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_CENTERMOUSE" unsafe prim_dS_CENTERMOUSE :: IO (Word32)
-dS_CONTEXTHELP :: WindowStyle
-dS_CONTEXTHELP =
-  unsafePerformIO(
-    prim_dS_CONTEXTHELP
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_CONTEXTHELP" unsafe prim_dS_CONTEXTHELP :: IO (Word32)
-dS_CONTROL :: WindowStyle
-dS_CONTROL =
-  unsafePerformIO(
-    prim_dS_CONTROL
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_CONTROL" unsafe prim_dS_CONTROL :: IO (Word32)
-dS_FIXEDSYS :: WindowStyle
-dS_FIXEDSYS =
-  unsafePerformIO(
-    prim_dS_FIXEDSYS
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_FIXEDSYS" unsafe prim_dS_FIXEDSYS :: IO (Word32)
-dS_LOCALEDIT :: WindowStyle
-dS_LOCALEDIT =
-  unsafePerformIO(
-    prim_dS_LOCALEDIT
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_LOCALEDIT" unsafe prim_dS_LOCALEDIT :: IO (Word32)
-dS_MODALFRAME :: WindowStyle
-dS_MODALFRAME =
-  unsafePerformIO(
-    prim_dS_MODALFRAME
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_MODALFRAME" unsafe prim_dS_MODALFRAME :: IO (Word32)
-dS_NOFAILCREATE :: WindowStyle
-dS_NOFAILCREATE =
-  unsafePerformIO(
-    prim_dS_NOFAILCREATE
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_NOFAILCREATE" unsafe prim_dS_NOFAILCREATE :: IO (Word32)
-dS_NOIDLEMSG :: WindowStyle
-dS_NOIDLEMSG =
-  unsafePerformIO(
-    prim_dS_NOIDLEMSG
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_NOIDLEMSG" unsafe prim_dS_NOIDLEMSG :: IO (Word32)
-dS_SETFONT :: WindowStyle
-dS_SETFONT =
-  unsafePerformIO(
-    prim_dS_SETFONT
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_SETFONT" unsafe prim_dS_SETFONT :: IO (Word32)
-dS_SETFOREGROUND :: WindowStyle
-dS_SETFOREGROUND =
-  unsafePerformIO(
-    prim_dS_SETFOREGROUND
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_SETFOREGROUND" unsafe prim_dS_SETFOREGROUND :: IO (Word32)
-dS_SYSMODAL :: WindowStyle
-dS_SYSMODAL =
-  unsafePerformIO(
-    prim_dS_SYSMODAL
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dS_SYSMODAL" unsafe prim_dS_SYSMODAL :: IO (Word32)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_3DLOOK" dS_3DLOOK :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_ABSALIGN" dS_ABSALIGN :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_CENTER" dS_CENTER :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_CENTERMOUSE" dS_CENTERMOUSE :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_CONTEXTHELP" dS_CONTEXTHELP :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_CONTROL" dS_CONTROL :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_FIXEDSYS" dS_FIXEDSYS :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_LOCALEDIT" dS_LOCALEDIT :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_MODALFRAME" dS_MODALFRAME :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_NOFAILCREATE" dS_NOFAILCREATE :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_NOIDLEMSG" dS_NOIDLEMSG :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_SETFONT" dS_SETFONT :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_SETFOREGROUND" dS_SETFOREGROUND :: WindowStyle
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dS_SYSMODAL" dS_SYSMODAL :: WindowStyle
 
-dM_GETDEFID :: WindowMessage
-dM_GETDEFID =
-  unsafePerformIO(
-    prim_dM_GETDEFID
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dM_GETDEFID" unsafe prim_dM_GETDEFID :: IO (Word32)
-dM_REPOSITION :: WindowMessage
-dM_REPOSITION =
-  unsafePerformIO(
-    prim_dM_REPOSITION
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dM_REPOSITION" unsafe prim_dM_REPOSITION :: IO (Word32)
-dM_SETDEFID :: WindowMessage
-dM_SETDEFID =
-  unsafePerformIO(
-    prim_dM_SETDEFID
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_dM_SETDEFID" unsafe prim_dM_SETDEFID :: IO (Word32)
-wM_CTLCOLORDLG :: WindowMessage
-wM_CTLCOLORDLG =
-  unsafePerformIO(
-    prim_wM_CTLCOLORDLG
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_wM_CTLCOLORDLG" unsafe prim_wM_CTLCOLORDLG :: IO (Word32)
-wM_CTLCOLORMSGBOX :: WindowMessage
-wM_CTLCOLORMSGBOX =
-  unsafePerformIO(
-    prim_wM_CTLCOLORMSGBOX
-    >>= \  res1  ->
-    (return (res1)))
-foreign import  ccall "prim_wM_CTLCOLORMSGBOX" unsafe prim_wM_CTLCOLORMSGBOX :: IO (Word32)
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dM_GETDEFID" dM_GETDEFID :: WindowMessage
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dM_REPOSITION" dM_REPOSITION :: WindowMessage
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_dM_SETDEFID" dM_SETDEFID :: WindowMessage
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_wM_CTLCOLORDLG" wM_CTLCOLORDLG :: WindowMessage
+foreign import  ccall unsafe "Win32Dialogue_stub_ffi.h prim_wM_CTLCOLORMSGBOX" wM_CTLCOLORMSGBOX :: WindowMessage
 
 ----------------------------------------------------------------
 -- End
