@@ -81,7 +81,7 @@ void* prim_drawMenuBar(HWND arg1)
 int access_prim_drawMenuBar_gc_failed(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_drawMenuBar_gc_failstring(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_enableMenuItem(HMENU arg1,UINT arg2,UINT arg3)
-{ static struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+{ static struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   UINT res1;int gc_failed;
 	    char* gc_failstring;
   do { DWORD res1 = EnableMenuItem(arg1,arg2,arg3);
@@ -93,9 +93,9 @@ void* prim_enableMenuItem(HMENU arg1,UINT arg2,UINT arg3)
       
       return(&gc_result);} while(0);
 }
-UINT access_prim_enableMenuItem_res1(void *ptr){ return(((struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res1);}
-int access_prim_enableMenuItem_gc_failed(void *ptr){ return(((struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_enableMenuItem_gc_failstring(void *ptr){ return(((struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+UINT access_prim_enableMenuItem_res1(void *ptr){ return(((struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res1);}
+int access_prim_enableMenuItem_gc_failed(void *ptr){ return(((struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_enableMenuItem_gc_failstring(void *ptr){ return(((struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 UINT prim_gMDI_USEDISABLED()
 { UINT res1;
   do {res1=GMDI_USEDISABLED;
@@ -498,7 +498,7 @@ HMENU prim_getMenu(HWND arg1)
       return((HMENU)(res1));} while(0);
 }
 void* prim_getMenuDefaultItem(HMENU arg1,int arg2,UINT arg3)
-{ static struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+{ static struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   UINT res1;int gc_failed;
 	    char* gc_failstring;
   do { int res1 = GetMenuDefaultItem(arg1,arg2,arg3);
@@ -510,11 +510,11 @@ void* prim_getMenuDefaultItem(HMENU arg1,int arg2,UINT arg3)
       
       return(&gc_result);} while(0);
 }
-UINT access_prim_getMenuDefaultItem_res1(void *ptr){ return(((struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res1);}
-int access_prim_getMenuDefaultItem_gc_failed(void *ptr){ return(((struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_getMenuDefaultItem_gc_failstring(void *ptr){ return(((struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+UINT access_prim_getMenuDefaultItem_res1(void *ptr){ return(((struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res1);}
+int access_prim_getMenuDefaultItem_gc_failed(void *ptr){ return(((struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_getMenuDefaultItem_gc_failstring(void *ptr){ return(((struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_getMenuState(HMENU arg1,UINT arg2,UINT arg3)
-{ static struct {HsWord res;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+{ static struct {HsWord32 res;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   UINT res;int gc_failed;
 	   char* gc_failstring;
   do { UINT res = GetMenuState(arg1,arg2,arg3);
@@ -526,9 +526,9 @@ void* prim_getMenuState(HMENU arg1,UINT arg2,UINT arg3)
       
       return(&gc_result);} while(0);
 }
-UINT access_prim_getMenuState_res(void *ptr){ return(((struct {HsWord res;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res);}
-int access_prim_getMenuState_gc_failed(void *ptr){ return(((struct {HsWord res;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_getMenuState_gc_failstring(void *ptr){ return(((struct {HsWord res;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+UINT access_prim_getMenuState_res(void *ptr){ return(((struct {HsWord32 res;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res);}
+int access_prim_getMenuState_gc_failed(void *ptr){ return(((struct {HsWord32 res;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_getMenuState_gc_failstring(void *ptr){ return(((struct {HsWord32 res;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 HMENU prim_getSubMenu(HMENU arg1,UINT arg2)
 { HMENU res1;
   do {res1 = GetSubMenu(arg1, arg2);
@@ -566,7 +566,7 @@ int access_prim_getMenuItemCount_res1(void *ptr){ return(((struct {HsInt res1;Hs
 int access_prim_getMenuItemCount_gc_failed(void *ptr){ return(((struct {HsInt res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_getMenuItemCount_gc_failstring(void *ptr){ return(((struct {HsInt res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_getMenuItemID(HMENU arg1,UINT arg2)
-{ static struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+{ static struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   UINT res1;int gc_failed;
 	    char* gc_failstring;
   do { UINT res1 = GetMenuItemCount(arg1);
@@ -578,11 +578,11 @@ void* prim_getMenuItemID(HMENU arg1,UINT arg2)
       
       return(&gc_result);} while(0);
 }
-UINT access_prim_getMenuItemID_res1(void *ptr){ return(((struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res1);}
-int access_prim_getMenuItemID_gc_failed(void *ptr){ return(((struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_getMenuItemID_gc_failstring(void *ptr){ return(((struct {HsWord res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+UINT access_prim_getMenuItemID_res1(void *ptr){ return(((struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res1);}
+int access_prim_getMenuItemID_gc_failed(void *ptr){ return(((struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_getMenuItemID_gc_failstring(void *ptr){ return(((struct {HsWord32 res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_unravelItemInfo(void * arg1)
-{ static struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;} gc_result;
+{ static struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;} gc_result;
   do { MENUITEMINFO* res1=arg1;
       gc_result.gc_res1 = (UINT)(res1->fType);
       gc_result.gc_res2 = (UINT)(res1->fState);
@@ -596,15 +596,15 @@ void* prim_unravelItemInfo(void * arg1)
       
       return(&gc_result);} while(0);
 }
-UINT access_prim_unravelItemInfo_gc_res1(void *ptr){ return(((struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res1);}
-UINT access_prim_unravelItemInfo_gc_res2(void *ptr){ return(((struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res2);}
-UINT access_prim_unravelItemInfo_gc_res3(void *ptr){ return(((struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res3);}
-HMENU access_prim_unravelItemInfo_gc_res4(void *ptr){ return(((struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res4);}
-HBITMAP access_prim_unravelItemInfo_gc_res5(void *ptr){ return(((struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res5);}
-HBITMAP access_prim_unravelItemInfo_gc_res6(void *ptr){ return(((struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res6);}
-DWORD access_prim_unravelItemInfo_gc_res7(void *ptr){ return(((struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res7);}
-char * access_prim_unravelItemInfo_gc_res9(void *ptr){ return(((struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res9);}
-int access_prim_unravelItemInfo_gc_res10(void *ptr){ return(((struct {HsWord gc_res1;HsWord gc_res2;HsWord gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res10);}
+UINT access_prim_unravelItemInfo_gc_res1(void *ptr){ return(((struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res1);}
+UINT access_prim_unravelItemInfo_gc_res2(void *ptr){ return(((struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res2);}
+UINT access_prim_unravelItemInfo_gc_res3(void *ptr){ return(((struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res3);}
+HMENU access_prim_unravelItemInfo_gc_res4(void *ptr){ return(((struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res4);}
+HBITMAP access_prim_unravelItemInfo_gc_res5(void *ptr){ return(((struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res5);}
+HBITMAP access_prim_unravelItemInfo_gc_res6(void *ptr){ return(((struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res6);}
+DWORD access_prim_unravelItemInfo_gc_res7(void *ptr){ return(((struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res7);}
+char * access_prim_unravelItemInfo_gc_res9(void *ptr){ return(((struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res9);}
+int access_prim_unravelItemInfo_gc_res10(void *ptr){ return(((struct {HsWord32 gc_res1;HsWord32 gc_res2;HsWord32 gc_res3;HsPtr gc_res4;HsPtr gc_res5;HsPtr gc_res6;HsWord32 gc_res7;HsPtr gc_res9;HsInt gc_res10;}*) ptr)->gc_res10);}
 void* prim_mallocMenuItemInfo()
 { static struct {HsPtr mp;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   MENUITEMINFO* mp;int gc_failed;
@@ -698,27 +698,27 @@ void * access_prim_getMenuItemInfo_gc_res2(void *ptr){ return(((struct {HsPtr gc
 int access_prim_getMenuItemInfo_gc_failed(void *ptr){ return(((struct {HsPtr gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_getMenuItemInfo_gc_failstring(void *ptr){ return(((struct {HsPtr gc_res2;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 void* prim_getMenuItemRect(HWND arg1,HMENU arg2,UINT arg3)
-{ static struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
+{ static struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
   char* gc_failstring;
   do { RECT r; BOOL success = GetMenuItemRect(arg1,arg2,arg3,&r);
       if ((gc_failed = ( !success ))) {gc_failstring =  ErrorString("GetMenuItemRect")  ;}
       else {gc_failed = 0;}
-      gc_result.gc_res2 = (LONG)((r).left);
-      gc_result.gc_res4 = (LONG)((r).top);
-      gc_result.gc_res6 = (LONG)((r).right);
-      gc_result.gc_res8 = (LONG)((r).bottom);
+      gc_result.gc_res1 = (LONG)((r).left);
+      gc_result.gc_res2 = (LONG)((r).top);
+      gc_result.gc_res3 = (LONG)((r).right);
+      gc_result.gc_res4 = (LONG)((r).bottom);
       gc_result.gc_failed = gc_failed;
       gc_result.gc_failstring = gc_failstring;
       
       return(&gc_result);} while(0);
 }
-LONG access_prim_getMenuItemRect_gc_res2(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
-LONG access_prim_getMenuItemRect_gc_res4(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
-LONG access_prim_getMenuItemRect_gc_res6(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res6);}
-LONG access_prim_getMenuItemRect_gc_res8(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res8);}
-int access_prim_getMenuItemRect_gc_failed(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
-void* access_prim_getMenuItemRect_gc_failstring(void *ptr){ return(((struct {HsInt gc_res2;HsInt gc_res4;HsInt gc_res6;HsInt gc_res8;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
+LONG access_prim_getMenuItemRect_gc_res1(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res1);}
+LONG access_prim_getMenuItemRect_gc_res2(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res2);}
+LONG access_prim_getMenuItemRect_gc_res3(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res3);}
+LONG access_prim_getMenuItemRect_gc_res4(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_res4);}
+int access_prim_getMenuItemRect_gc_failed(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
+void* access_prim_getMenuItemRect_gc_failstring(void *ptr){ return(((struct {HsInt32 gc_res1;HsInt32 gc_res2;HsInt32 gc_res3;HsInt32 gc_res4;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
 int prim_hiliteMenuItem(HWND arg1,HMENU arg2,UINT arg3,UINT arg4)
 { do { BOOL success = HiliteMenuItem(arg1,arg2,arg3,arg4);
       
@@ -754,8 +754,9 @@ void* prim_loadMenu(HINSTANCE arg1,LPCSTR arg2)
 HMENU access_prim_loadMenu_res1(void *ptr){ return(((struct {HsPtr res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->res1);}
 int access_prim_loadMenu_gc_failed(void *ptr){ return(((struct {HsPtr res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_loadMenu_gc_failstring(void *ptr){ return(((struct {HsPtr res1;HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
-UINT prim_menuItemFromPoint(HWND arg1,HMENU arg2,LONG gc_arg3,LONG gc_arg5)
+UINT prim_menuItemFromPoint(HWND arg1,HMENU arg2,LONG gc_arg2,LONG gc_arg3)
 { UINT res1;POINT arg3;
+  (arg3).x = (LONG)gc_arg2; (arg3).y = (LONG)gc_arg3;
   do { UINT res1=MenuItemFromPoint(arg1,arg2,arg3);
       
       return((UINT)(res1));} while(0);
@@ -832,10 +833,11 @@ void* prim_setMenuItemInfo(HMENU arg1,UINT arg2,int arg3,UINT arg4,void * arg5)
 }
 int access_prim_setMenuItemInfo_gc_failed(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failed);}
 void* access_prim_setMenuItemInfo_gc_failstring(void *ptr){ return(((struct {HsInt gc_failed;HsPtr gc_failstring;}*) ptr)->gc_failstring);}
-void* prim_trackPopupMenu(HMENU arg1,UINT arg2,int arg3,int arg4,HWND arg5,LONG gc_arg3,LONG gc_arg5,LONG gc_arg7,LONG gc_arg9)
+void* prim_trackPopupMenu(HMENU arg1,UINT arg2,int arg3,int arg4,HWND arg5,LONG gc_arg2,LONG gc_arg3,LONG gc_arg4,LONG gc_arg5)
 { static struct {HsInt gc_failed;HsPtr gc_failstring;} gc_result;
   int gc_failed;
   char* gc_failstring;RECT arg6;
+  (arg6).left = (LONG)gc_arg2; (arg6).top = (LONG)gc_arg3; (arg6).right = (LONG)gc_arg4; (arg6).bottom = (LONG)gc_arg5;
   do { BOOL success = TrackPopupMenu(arg1,arg2,arg3,arg4,0,arg5,&arg6);
       if ((gc_failed = ( !success ))) {gc_failstring =  ErrorString("TrackPopupMenu")  ;}
       else {gc_failed = 0;}
