@@ -1,3 +1,7 @@
+#include <windows.h>
+#include "errors.h"
+#include "win32debug.h"
+#include "finalizers.h"
 #include "HsFFI.h"
 extern void* prim_beginPath(HDC arg1);
 extern int access_prim_beginPath_gc_failed(HsPtr);
@@ -27,4 +31,4 @@ extern int access_prim_strokePath_gc_failed(HsPtr);
 extern void* access_prim_strokePath_gc_failstring(HsPtr);
 extern void* prim_widenPath(HDC arg1);
 extern int access_prim_widenPath_gc_failed(HsPtr);
-extern void* access_prim_widenPath_gc_failstring(HsPtr);
\ No newline at end of file
+extern void* access_prim_widenPath_gc_failstring(HsPtr);
