@@ -256,6 +256,7 @@ type ClauseQ = Q Clause
 type BodyQ = Q Body
 type StmtQ = Q Stmt
 type RangeQ = Q Range
+type FieldExpQ      = Q FieldExp
 
 --runE :: ExpQ -> Exp
 --runE x = runQ 0 x
@@ -816,8 +817,8 @@ pprVarStrictType (v, str, t) = text v <+> text "::" <+> pprStrictType (str, t)
 
 ------------------------------
 pprStrictType :: (Strict, Type) -> Doc
-pprStrictType (IsStrict, t) = char '!' <> pprType t
-pprStrictType (NotStrict, t) = pprType t
+pprStrictType (IsStrict, t) = char '!' <> pprParendType t
+pprStrictType (NotStrict, t) = pprParendType t
 
 ------------------------------
 pprParendType :: Type -> Doc
